#!/usr/bin/env python

########################################################################
# CLAW3 Filebrowser
# Copyright 2004, Gabe Ginorio <gabe@zevallos.com.br>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
######################################################################

# These are python modules
import os, os.path, sys, string, textwrap, re, shutil, stat, time, commands, glob, mimetypes, signal, fcntl
from xml.dom import minidom

# These are pygtk modules and custom modules
import pygtk
pygtk.require("2.0")

import gtk, gtk.gdk, pango, gobject, gtk.gdk

# Add some paths for importing modules (subclasses)
basepath = string.replace(os.path.realpath(sys.argv[0]),"Claw3.py","")
for directory in ["/modules"]:
	if not basepath + directory in sys.path:
		sys.path.append(basepath + directory)

# These are not dialogs
import mimereader, xmlreader, fstabreader, mtabreader, notify
from eyeD3 import *

# These are dialogs
import preferencesetter, choose, progressdialog, overwrite, rename, waiting

class Claw3:

	################################################################
	# This is called when the class is first called (i.e. the program is run)
	################################################################	
	def __init__(self):

		# We must initialize from variables we will use often
		self.programPath = os.path.abspath(string.replace(sys.argv[0], "Claw3.py", "")) + "/"
		self.imagePath = self.programPath + "images/"

		# Update the themes in ~/.themes
		# Create the .themes directory if it doesn't exist
		if not os.path.exists(os.environ['HOME'] + "/.themes"):
			os.mkdir(os.environ['HOME'] + "/.themes")
			
		# Compare the installed and the themes that came with the package
		installedThemes = os.listdir(os.environ['HOME'] + "/.themes/")
		packageThemes = os.listdir(self.programPath + "themes/")
		notInstalled = filter(lambda theme: theme not in installedThemes, packageThemes)

		# Install and directories that don't exist
		if notInstalled != []:
			for theme in notInstalled:
				shutil.copytree(self.programPath + "/themes/" + theme, os.environ['HOME'] + "/.themes/" + theme) 

		# This section set's the style (if a ~/.gtkrc file exists)
		if os.path.exists(os.environ['HOME'] + "/.gtkrc"):

			# Open the .gtkrc file
			rcFile = file(os.environ['HOME'] + "/.gtkrc", 'r').readlines()
			
			# Find the line that points to a theme 
			line = filter(lambda line : re.findall("/gtk", line), rcFile)
			
			# If there is no "gtk" in any line, don't parse it at all
			if line == []:
			
				# Set a default theme
				line = ["include \"" + os.environ['HOME'] + "/.themes/MacOS-X/gtk-2.0/gtkrc\""]
			
			# Change any gtk/ to gtk-2.0/
			if not re.findall("gtk-2.0", line[0]):
				line = string.replace(line[0], "/gtk/", "/gtk-2.0/")
				
			# Open up the file for writing
			rcFile = file(os.environ['HOME'] + "/.gtkrc", 'w')
			
			# Write the information to the file
			rcFile.write("# This is the default gtk theme file. Do Not Edit Manually!\n")
			rcFile.write(line[0] + "\n")
			rcFile.write("# End of gtk theme file\n")

			try:
				# Parse the ~/.gtkrc file
				gtk.rc_parse(os.environ['HOME'] + "/.gtkrc")
			except:
				# Let the user know if there is an error if something went wrong
				print "I can only use gtk-2.0 themes, not gtk"

		# If a ~/.gtkrc file doesn't exist, then we must create one
		else:
			# Open up the file for writing
			rcFile = file(os.environ['HOME'] + "/.gtkrc", 'w')
			
			# Write the information to the file
			rcFile.write("# This is the default gtk theme file. Do Not Edit Manually!\n")
			rcFile.write("include \"" + os.environ['HOME'] +"/.themes/MacOS-X/gtk-2.0/gtkrc\"")
			rcFile.write("# End of gtk theme file\n")
			
	
		# This lets the resize function not to resize while listing 
		# Otherwise it loops every time an icon appears then crashes
		self.listing = 1

		# These are for the back button
		self.history = [[], []]
		self.backing = 0
		
		# This holds the notifier classes (for "dnotify")
		self.notifier = [0,0]

		# This keeps track of the pane we are over
		self.layoutOver = 0

		# This keeps track of the size of the layouts
		self.width = [0,0]

		# This will be used for listing (otherwise you can't keep them in order)
		self.eventBoxes = [[],[]]

		# This initializes the self.users variable (holds user list)
		self.getUsers()

		# This gets the mountpoints in  /etc/fstab and the mounted points in /etc/mtab
		self.mountpoints = fstabreader.read()
		self.mountedpoints = mtabreader.read()

		# Here are some default preferences
		self.termType		= "xterm"
		self.clearAddress	= 0
		self.previewImage	= "on"
		self.previewAudio	= "off"
		self.showPermissions = "yes"
		self.archiver		= "bzip2"
		self.font			= pango.FontDescription("sans 12")
		self.fontColor		= gtk.gdk.Color(0,0,0,0)
		self.background		= "default"
		self.layoutColor		= gtk.gdk.Color(65000, 65000, 65000, 0)
		self.language 		= "english"

		# Now, lets parce the preferences file
		xmlreader.getPreferences(self)
		xmlreader.getLanguage(self, self.language)

		# Find the mimetype files that don't exist in the ~/.claw directory
		files = ["application","audio","image","text","video","types"]
		files = filter(lambda x : not os.path.exists(os.environ['HOME'] + "/.claw/mime." + x), files)
		
		# For any mimetype files that don't exist, copy them from the program mimetypes directory
		map(lambda x : shutil.copy(self.programPath + "mimetypes/mime." + x, os.environ['HOME'] + "/.claw/"), files)
		
		# Update any installed mime files
		mimereader.update(self)

		# Now add the file types in mime.types to the python path
		mimetypes.init([os.environ['HOME'] + "/.claw/mime.types", ] + mimetypes.knownfiles)

		# Set all the icons we will use
		self.pixbufs = {}
		icons = os.listdir(self.imagePath + "icon")
		for icon in icons:
			self.pixbufs[icon] = gtk.gdk.pixbuf_new_from_file(self.imagePath +"icon/" + icon)

		# Set the current directory to whereever we were called from
		self.currentDirectory = [os.getcwd(), os.getcwd()]

		# Make sure this variable is there, but empty
		self.currentContents = [[], []]

		# Set the current icon window to the top one
		self.activeWindow = 0

		# First we create a main window to hold all the other widgets (we connect the corner 'X' to mainquit)
		self.mainWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.mainWindow.set_title("CLAW3 Filebrowser")
		self.mainWindow.connect("destroy", gtk.mainquit)
		self.mainWindow.resize(650, 500)
		self.mainWindow.show()

		# We next make the icon for the iconified claw
		icon = gtk.gdk.pixbuf_new_from_file(self.imagePath + "other/SurferTux.png")
		self.mainWindow.set_icon(icon)

		# Next we create a table to allow formatting of the elements within the main window
		mainTable = gtk.Table(gtk.FALSE)
		mainTable.show()
		self.mainWindow.add(mainTable)

		# Make the bottom bar next
		self.makeBottomBar(mainTable)

		# We then call a function to create the toolbar
		self.makeToolbar(mainTable)

		# Make the address bar
		self.makeAddressBar(mainTable)

		# We next call a function to create the panes and the icon window
		self.makeIconView(mainTable)

		# Now, we call a function to create the sidebar
		self.makeSideBar()

		# Make the Bookmarks Browser
		self.makeBookmarkBrowser()

		# Make the Quick System Browser
		self.makeQuickSystemBrowser()

		# Lastly, we call the function to list the contents of the current directory
		self.listCurrentDirectory()
		
		# Redraw the widget using the theme
		self.mainWindow.reset_rc_styles()
	

	################################################################
	# This function creates the bottombar and handles it's functions (callbacks)
	################################################################
	def makeBottomBar(self, mainTable):

		# Create a box to hold the infobar
		infoBox = gtk.HBox()
		infoBox.show()
		mainTable.attach(infoBox, 0,1,3,4, gtk.EXPAND, gtk.SHRINK)
		
		# Create a box for the dual pane button
		buttonBox = gtk.HButtonBox()
		buttonBox.show()
		buttonBox.set_layout(gtk.BUTTONBOX_END)
		mainTable.attach(buttonBox, 1,2,3,4, gtk.SHRINK|gtk.FILL, gtk.SHRINK)


		# We create at this point the infamous infobar (let's us know what is going on)
		self.infoBar = gtk.Label()
		self.infoBar.show()
		self.infoBar.set_text("Chris's Lame File Browser is Ready")
		self.infoBar.modify_font(pango.FontDescription("sans 12"))
		infoBox.pack_start(self.infoBar, gtk.FALSE, gtk.TRUE)

		# Create a button to activate the second icon view
		#paneToggleButton = gtk.ToggleButton()
		paneToggleButton = self.makeButton("toggle", None, "clicked", self.toggleButton)		
		paneToggleButton.show()
		#paneToggleButton.connect("clicked", self.toggleButton)
		buttonBox.pack_start(paneToggleButton, gtk.FALSE, gtk.TRUE)

		# Create a tooltip for the button
		toggleTooltip = gtk.Tooltips()
		toggleTooltip.enable()
		toggleTooltip.set_tip(paneToggleButton, "Toggle Lower Pane", tip_private=None)
		# tooltip = gtk.tooltips_data_get(paneToggleButton)

		# Create an image for the button
		toggleImage = gtk.Image()
		toggleImage.show()
		toggleImage.set_from_file(self.imagePath + "other/toggle_pane.png")
		paneToggleButton.add(toggleImage)


	######################################
	# This function is the callback for toggle pane
	######################################
	def toggleButton(self, button):

		# If the visible property is false, show it
		if not self.secondScroll.get_property('visible'):
			self.secondScroll.show()

			# Then list the current directory for the second pane
			self.activeWindow = 1
			if len(self.eventBoxes) != 0:
				self.removeOldIcons()
			self.listCurrentDirectory()
		else:
		# Otherwise, hide the second scrollbox and set the active window to the top one
			self.secondScroll.hide()			
			self.activeWindow = 0


	################################################################
	# This function creates the toolbar and handles it's functions (callbacks)
	################################################################
	def makeToolbar(self, mainTable):
		
		# Create a handle box for the toolbar
		self.toolbarHandleBox = gtk.HandleBox()
		self.toolbarHandleBox.show()
		mainTable.attach(self.toolbarHandleBox, 0,2,0,1, gtk.FILL, gtk.FILL)

		# Make a toolbar and format it
		self.mainToolbar = gtk.Toolbar()
		self.mainToolbar.show()
		self.mainToolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
		#self.mainToolbar.set_style(gtk.TOOLBAR_ICONS)
		self.mainToolbar.set_style(gtk.TOOLBAR_BOTH)
		self.mainToolbar.set_tooltips(gtk.TRUE)

		# Create the information for making the toolbar buttons and funtions
		toolNames =	[self.messages["back"][0],self.messages["up"][0], self.messages["reload_directory"][0],self.messages["home"][0], 
						self.messages["xterm"][0], self.messages["new_browser"][0], self.messages["preferences"][0],
						self.messages["empty_trash"][0]]
		toolTips     =	[self.messages["back"][1],self.messages["up"][1], self.messages["reload_directory"][1],self.messages["home"][1], 
						self.messages["xterm"][1], self.messages["new_browser"][1], self.messages["preferences"][1],
						self.messages["empty_trash"][1]]
		iconNames=	["back.png", "up.png", "reload.png", "home.png", "terminal.png", "new.png", "preferences.png", "trash.png"]
		callBacks   = 	[self.goBack, self.goUp, self.goReload, self.goHome, self.openTerm, self.newBrowser, 
					self.openPreferences, self.emptyTrash]
		buttonType=  ["button", "button", "button","button", "button", "button", "button", "button"]

		#Create the toolbar items and add them to the toolbar
		for x in range(len(toolNames)):

			# We create a pixbuf from the picture (make a copy in memory)
			iconPixbuf = gtk.gdk.pixbuf_new_from_file(self.imagePath + "toolbar/" + iconNames[x])
			
			# Make the icons small
			iconPixbuf = iconPixbuf.scale_simple(22, 22, gtk.gdk.INTERP_HYPER)

			# Create an image from the pixbuf
			image = gtk.Image()
			image.set_from_pixbuf(iconPixbuf)

			if buttonType[x] == "button":
				self.mainToolbar.append_item(toolNames[x], toolTips[x], "CLAW3", image, callBacks[x], None)
			#elif buttonType[x] == "toggle":
			#	self.mainToolbar.append_element(gtk.TOOLBAR_CHILD_TOGGLEBUTTON, None, toolNames[x], 
			#								toolTips[x], "iRoq", image, callBacks[x], user_data=None)	

		# Add the toolbar to the handle bar
		self.toolbarHandleBox.add(self.mainToolbar)


	######################################
	# This function is the callback for goBack
	######################################
	def goBack(self, widget):
		
		# First, let the list function know we are going back
		self.backing = 1

		# Rid ourselves of the current entry in the histroy list (if it exists)
		if len(self.history[self.activeWindow]) > 1: 
			self.history[self.activeWindow].pop()

			# Then we set the current directory to the last item on the list (where we were at before here)
			self.currentDirectory[self.activeWindow]= self.history[self.activeWindow][len(self.history[self.activeWindow])-1]

			# List the new directory
			self.removeOldIcons()
			self.listCurrentDirectory()		
	

	######################################
	# This function is the callback for goUp
	######################################
	def goUp(self, widget):
		
		# If we have permission, and the directory exists
		if self.currentDirectory[self.activeWindow]!= "/" and os.access(os.path.dirname(self.currentDirectory[self.activeWindow]), os.R_OK):

			# Go up one directory
			self.currentDirectory[self.activeWindow] = os.path.dirname(self.currentDirectory[self.activeWindow]) 

			# Relist
			self.removeOldIcons()
			self.listCurrentDirectory()


	######################################
	# This function is the callback for goReload
	######################################
	def goReload(self, widget=None):

		# Remove the old icons
		self.removeOldIcons()

		# Let's get the files in the current directory
		files = os.listdir(self.currentDirectory[self.activeWindow])

		# If files have been added, then we have to relist
		if len(files) > len(self.eventBoxes[self.activeWindow]):
			self.listCurrentDirectory()
			return 0

		# Check to see if the location of an icon isn't in the directory listing
		gone = []
		for icon in self.currentContents[self.activeWindow]:
			if os.path.basename(self.currentContents[self.activeWindow][icon][2]) not in files:
				gone.append(icon)
				
		# Delete any missing files from the icon list
		if gone != []:
			for icon in gone:
				del self.currentContents[self.activeWindow][icon]
				self.eventBoxes[self.activeWindow].remove(icon)

		# Re-place them
		self.placeIcons()


	######################################
	# This function is the callback for goHome
	######################################
	def goHome(self, widget):

		# Change the current Directory to HOME
		self.currentDirectory[self.activeWindow]= os.environ['HOME']
		
		# Relist
		self.removeOldIcons()
		self.listCurrentDirectory()


	######################################
	# This function is the callback for openTerm
	######################################
	def openTerm(self, widget):
		
		# Open a terminal (whatever is specified)
		os.popen2(self.termType)


	######################################
	# This function is the callback for openTerm
	######################################
	def newBrowser(self, widget):
		
		# Open a new CLAW3
		os.popen2(self.programPath + "Claw3.py")


	######################################
	# This function is the callback for open preferences dialog
	######################################
	def openPreferences(self, widget):

		# Open the preferences dialog
		preferencesetter.Dialog(self)	


	######################################
	# This function is the callback for emptying ~/..Wastbasket
	######################################
	def emptyTrash(self, widget):

		# Get a list of the files in the trash (if we can)
		try:
			trashFiles = os.listdir(os.environ['HOME'] + "/..Wastebasket")
		except:
			# If not spit out an informative error
			self.infoBar.set_text("Cannot list ~/..Wastebasket. Make sure it exists, and check permissions.")
			return 1


		# Use the deleteFile function to erase the files (if it is possible)
		if len(trashFiles) != 0:
			try:
				map(lambda x : self.deleteFile(os.environ['HOME'] + "/..Wastebasket/" + x), trashFiles)
				self.infoBar.set_text("Trash is Empty")
			except:
				# If we can't empty it, it could be permissions
				self.infoBar.set_text("I can't delete the contents of ..Wastebasket. Check Permissions.")
				return 1
		else:
			# If there is nothing in the trash, let the user know
			self.infoBar.set_text("Nothing in Trash.")
			return 1


	########################################################################
	#					MAKE PATH ENTRY BOX
	########################################################################	
	def makeAddressBar(self, mainTable):

		# Make a combo box and add it to the main table
		self.mainCombo = gtk.Combo()
		self.mainCombo.show()
		mainTable.attach(self.mainCombo , 0,1,1,2, gtk.FILL|gtk.EXPAND, gtk.FILL, 5,5)

		# Connect some events to the combo box
		#self.mainCombo.set_events(gtk.gdk.KEY_PRESS_MASK)
		#self.mainCombo.entry.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		# This is the return key
		self.mainCombo.entry.connect("activate", self.goToDir)
		# This is for any other key
		self.mainCombo.entry.connect("key_press_event", self.autoCompleteList)

		# Create a button box and add it to the table
		self.buttonBox = gtk.HBox()
		mainTable.attach(self.buttonBox , 1,2,1,2, gtk.SHRINK|gtk.FILL,gtk.FILL, 5,5)

		# Make a GO button and add it to the button box
		self.goButton = self.makeButton("button", self.messages["go"][0], "clicked",  self.goToDir)			
		self.buttonBox.pack_start(self.goButton, gtk.FALSE, gtk.TRUE, 2)

		# Make a FILTER button and add it to the button box
		self.filterButton = self.makeButton("button", self.messages["filter"][0], "clicked", self.filterFiles)
		self.buttonBox.pack_start(self.filterButton, gtk.FALSE, gtk.TRUE, 2)

		# Make a SEARCH button and add it to the button box
		self.searchButton = self.makeButton("button", self.messages["search"][0], "clicked", self.findFiles)
		self.buttonBox.pack_start(self.searchButton, gtk.FALSE, gtk.TRUE, 2)

		# Make a CLEAR toggle button and add it to the box
		self.clearButton = self.makeButton("button", self.messages["clear"][0], "clicked", self.clearMe)
		self.buttonBox.pack_start(self.clearButton, gtk.FALSE, gtk.TRUE, 2)

		# Show all the buttons
		self.buttonBox.show_all()


	######################################
	# The callback for the <ENTER> key 
	######################################
	def goToDir(self, button):

		# Get the location from the box
		location = self.mainCombo.entry.get_text()

		# If the location is a directory, list it
		if os.path.isdir(location):
			if os.access(location, os.R_OK):
				self.currentDirectory[self.activeWindow]= location
				self.removeOldIcons()
				self.listCurrentDirectory()
			else:
				# If you don't have permission to list the directory, then don't 
				self.infoBar.set_text("You don't have permission to enter : " + location)
		else:
			# If location isn't a directory, then print the error in the info bar
			self.infoBar.set_text(location + " is not a directory")


	######################################
	# The callback for the FILTER button
	######################################
	def filterFiles(self, button):

		# First we need to get the pattern matches (using glob)
		search = self.mainCombo.entry.get_text()
		matches = glob.glob(search)

		# Clear the directory of icons		
		self.removeOldIcons()

		# Then we filter out addresses not on our match list
		removeThese = []
		for box in self.currentContents[self.activeWindow]:
			if os.path.basename(self.currentContents[self.activeWindow][box][2]) not in matches:

				# All non-matches go on this list
				removeThese.append(box)

		# We use the list to remove the icons we don't want
		for box in removeThese:	
			del self.currentContents[self.activeWindow][box]
			self.eventBoxes[self.activeWindow].remove(box)

		# Reload the directory with only the remaining icons
		self.placeIcons()


	######################################
	# The callback for the SEARCH toggle button
	######################################
	def findFiles(self, button):
	
		# Get the location from the box
		search = self.mainCombo.entry.get_text()
		
		# Do a locate for the file
		results = commands.getstatusoutput("locate " + self.returnClean(search))
		
		# Check to see if it failed
		if results[0] != 0:
			self.infoBar.set_text("No matches found using locate : try running updatedb as root")
			return 0

		# Make a list for the combo box
		results = string.split(results[1], "\n")
		
		# Concatenate the list so it isn't outrageously long
		if len(results) > 30:
			self.infoBar.set_text("Appended Results to 30")
			results = results[:30]
		
		# Add the list to the combo box
		self.mainCombo.set_popdown_strings(results)
			
		
	######################################
	# The callback for the CLEAR toggle button
	######################################
	def clearMe(self, button):
	
		# Reverse the value of self.clear
		if self.clearAddress == 0:
		
			# If it is toggled, erase the popdown list
			self.clearAddress = 1
			self.mainCombo.set_popdown_strings([""])
		else:
			self.clearAddress = 0


	######################################
	# The callback for any unprintable characters
	######################################
	def autoCompleteList(self, entry, event):		

		# If the non-printable character is <TAB> then move onto auto complete
		if event.keyval != 65289:
			return 0

		# If it is a <TAB>, and the entry box has something in it
		if len(self.mainCombo.entry.get_text()) > 0:

			# Get the text from the box
			path = self.mainCombo.entry.get_text()

			# Split the path from the partial entry
			beginning, end = os.path.split(path)

			# If the first half is a valid directory, then get a list of it's contents
			if os.path.isdir(os.path.abspath(beginning)):
				currentList = os.listdir(os.path.abspath(os.path.realpath(beginning)))

				# Make a variable to hold the contents of the current directory
				dirList= []

				# If there is nothing after the directory location, put everything in the list
				if end != []:
					for file in currentList:
						if re.findall("\A" + end, file) != []:
							if os.path.isdir(beginning + "/" + file):
								dirList.append(beginning + "/" + file + "/")						
				else:
					# If there is a partial file name, then add all matching patterns to the list
					for file in currentList:
						if os.path.isdir(file):
							dirList.append(beginning + "/" + file + "/")

				# If however, nothing was found, then list what is already there 
				# and inform the user			
				if dirList == []:
					dirList = [path]
					self.infoBar.set_text("No Matches Found")

				# Add the resulting list to the combo box lst		
				self.mainCombo.set_popdown_strings(dirList)

				# Reset the focus
				entry, button = self.mainCombo.get_children()
				entry.set_flags(gtk.CAN_FOCUS)			
				entry.grab_focus()
				entry.set_position(len(entry.get_text()))


	################################################################
	# This function creates the panes and the icon view
	################################################################
	def makeIconView(self, mainTable):
		
		# First we create a horizonital paned window
		self.mainPane = gtk.HPaned()
		self.mainPane.show()
		mainTable.attach(self.mainPane, 0, 2, 2, 3)

		# Secondly, we create a vertical pane for the icon views
		iconPane = gtk.VPaned()
		iconPane.show()
		self.mainPane.add2(iconPane)


		# Then we create scrolled windows for the layouts to go into
		mainScroll = [gtk.ScrolledWindow(), gtk.ScrolledWindow()]
		mainScroll[0].show()
		# mainScroll[1].show()
		self.secondScroll = mainScroll[1]

		# This sets the policy so you only see the scroll bars when they are needed
		mainScroll[0].set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		mainScroll[1].set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

		# These scrolled windows are added to the icon panes
		iconPane.add1(mainScroll[0])
		iconPane.add2(mainScroll[1])

		# Next, we create a layouts for the icons
		self.mainLayout = [gtk.Layout(), gtk.Layout()]
		self.mainLayout[0].show()
		self.mainLayout[1].show()

		# Set the background color
		self.mainLayout[0].modify_bg(gtk.STATE_NORMAL, self.layoutColor)
		self.mainLayout[1].modify_bg(gtk.STATE_NORMAL, self.layoutColor)

		# Connect the layouts to callbacks (resizing, button presses)
		self.mainLayout[0].connect("size-allocate", self.resizeLayout)
		self.mainLayout[1].connect("size-allocate", self.resizeLayout)
		self.mainLayout[0].connect("button-press-event", self.layoutClicked, 0)
		self.mainLayout[1].connect("button-press-event", self.layoutClicked, 1)

		# This sets the layouts as drag and drop targets
		targets = [ ("text/plain", 0, 0)]
		self.mainLayout[0].drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,  gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
		self.mainLayout[1].drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,  gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)

		# Connect the layouts to a drop handler
		self.mainLayout[0].connect("drag-data-received", self.iconDropped)
		self.mainLayout[1].connect("drag-data-received", self.iconDropped)

		# Add the main layout to the scrolled window
		mainScroll[0].add(self.mainLayout[0])
		mainScroll[1].add(self.mainLayout[1])	


	######################################
	# This function handles the callback if the layout is clicked
	######################################
	def layoutClicked(self, layout, event, number):

		self.activeWindow = int(number)
		if self.clearAddress	== 0:
			self.mainCombo.set_popdown_strings([self.currentDirectory[self.activeWindow]])

		# Let's make sure that we are not over an icon
		if self.over == 0 and event.button == 3:

			# We need to create a menu to give the user options
			menu = gtk.Menu()
			menu.show()


			# Create a select all item
			selectallItem = gtk.MenuItem(self.messages["select_all"][0])
			selectallItem.show()
			selectallItem.connect("activate",self.layoutMenu, "select all")
			menu.append(selectallItem)

			# Create a select all item
			invertItem = gtk.MenuItem(self.messages["invert_selection"][0])
			invertItem.show()
			invertItem.connect("activate",self.layoutMenu, "invert selection")
			menu.append(invertItem)

			# Create a paste option (about time)
			pasteItem = gtk.MenuItem("Paste")
			pasteItem.show()
			pasteItem.connect("activate",self.layoutMenu, "paste")
			menu.append(pasteItem)

			# Create a new folder option (I really need this, myself)
			folderItem = gtk.MenuItem(self.messages["create_new_folder"][0])
			folderItem.show()
			folderItem.connect("activate",self.layoutMenu, "folder")
			menu.append(folderItem)

			# Send selected to Trash option
			trashItem = gtk.MenuItem(self.messages["send_selected_files_to_trash"][0])
			trashItem.show()
			trashItem.connect("activate",self.layoutMenu, "trash")
			menu.append(trashItem)

			# Make the menu "Popup" immediately
			menu.popup(None, None, None, 0, 0)


	######################################
	# This function handles the callback if the layout is clicked
	######################################
	def layoutMenu(self, menuItem, selection):


		# Here is where we paste files
		if selection == "paste":
			if os.path.exists(os.environ['HOME'] + "/.claw/prettyclipboard.txt"):
					
				# First thing, do we have write permission in this directory
				if not os.access(self.currentDirectory[self.activeWindow], os.W_OK):
					self.infoBar.set_text("You don't have permission to paste here.")
					return 1

				# Read the files in the clipboard
				pasteFiles = file(os.environ['HOME'] + "/.claw/prettyclipboard.txt", 'r').readlines()

				# Remove the trailing new lines
				for x in range(len(pasteFiles)):
					pasteFiles[x] = string.strip(pasteFiles[x])

				# Find out if this is a copy or a cut
				pasteType = string.strip(pasteFiles[0])
				pasteFiles.remove(pasteType)

				# Make sure the file exist and we can read it
				for aFile in pasteFiles:
					if not os.path.exists(aFile) or not os.access(aFile, os.R_OK):
						# Remove any non existant files or files without copy permissions
						pasteFiles.remove(aFile)

					if pasteType == "cut" and not os.access(aFile, os.W_OK) and aFile in pasteFiles:
						# Remove any files we "cut" that don't have write access, if it exists
						pasteFiles.remove(aFile)
			

				# Make sure some files survived
				if pasteFiles == []:
					# If no files survived, let the user know and leave
					self.infoBar.set_text("Either you don't have permission to copy or the files on the clipboard don't exist.")
					return 0

				# Now, use the progress dialog to copy and selected files
				for aFile in pasteFiles:
					pg = progressdialog.fileMover(self, aFile, self.currentDirectory[self.activeWindow]+ "/" + os.path.basename(aFile))

				# If the files were cut, let's erase the source files
				if pasteType == "cut":
					for aFile in pasteFiles:

						# Let's use the delete file function, because it's cool (should move file to /dev/null)
						self.deleteFile(aFile)		

			else:
				# If the clipboard file is missing, let the user know, make a new one, then return
				self.infoBar.set_text("Your clipboard is missing ... I'll make another.")
				clipfile = file(os.environ['HOME'] + "/.claw/prettyclipboard.txt", 'w')
				clipfile.close()
				return 1

		# Let's create a new folder
		elif selection == "folder":

			# Let's see if we can write to the current Directory
			if not os.access(self.currentDirectory[self.activeWindow], os.W_OK):
				self.infoBar.set_text("You don't have permission to write to this directory")
				return 1

			# Now, let's make the new directory (like the name?)
			try:
				for x in range(100):
					if not os.path.exists(self.currentDirectory[self.activeWindow]+ "/a_new_folder_" + str(x)):
						os.mkdir(self.currentDirectory[self.activeWindow]+ "/a_new_folder_" + str(x))
						name = self.currentDirectory[self.activeWindow]+ "/a_new_folder_" + str(x)
						break
			except:
				# I have no idea why this would happen, but what the heck ... you never know
				self.infoBar.set_text("Error creating directory, and it isn't a permission problem.")
				return 1
			
			# We then call the rename dialog
			rename.Rename(self, [name])
			

		# This allows you to select all the icons at once
		elif selection  == "select all":

			# Set all icons to selected
			for icon in self.currentContents[self.activeWindow]:
				icon.set_state(gtk.STATE_SELECTED)

		# This allows you to invert your selection
		elif selection  == "invert selection":

			# Simply set the state to opposite of whatever it is now
			for icon in self.currentContents[self.activeWindow]:
				if icon.state == gtk.STATE_SELECTED:
					icon.set_state(gtk.STATE_NORMAL)
				else:
					icon.set_state(gtk.STATE_SELECTED)

		# This allows you to send all selected files to ~/.Wastebasket
		elif selection == "trash":
			
			# Get the selected icons
			selections = self.getSelected()

			# Check permissions and move the files
			for selection in selections:
				location = self.currentContents[self.activeWindow][selection][2]

				# Check for write permissions
				if not os.access(location, os.W_OK):

					# If you don't have write permissions, set the state back to normal
					selection.set_state(gtk.STATE_NORMAL)

					# Then let the user know and remove it from the list of selected icons
					self.infoBar.set_text("Can't Send " + os.path.basename(location) + "to ~/..Wastebasket")
					selections.remove[selection]

				# Otherwise, move the file
				else:

					# Let the user know what is happening
					self.infoBar.set_text("Moving " + os.path.basename(location))

					# Use the progress dialog to copy the files (then delete the old file)
					try:
						progressdialog.fileMover(self, location, os.environ['HOME'] + "/..Wastebasket/" + os.path.basename(location))

						self.deleteFile(os.path.basename(location))

					except:
						self.infoBar.set_text("Can't Move Files to " + os.environ['HOME'] + "/..Wastebasket")		


	################################################################
	# This function creates the side bar
	################################################################
	def makeSideBar(self):

		# First we create a notebook to hold the different windows
		self.mainNotebook = gtk.Notebook()
		self.mainNotebook.show()
		#mainNotebook.set_tab_pos(gtk.POS_LEFT)
		self.mainPane.add1(self.mainNotebook)

		# We then create two frames with boxes and add them to the notebook
		sideFrame = []
		sideFrame.append(gtk.Frame(self.messages["system_quick_browser"][0]))
		sideFrame[0].show()
		sideFrame.append(gtk.Frame(self.messages["bookmarks"][0]))
		sideFrame[1].show()

		# Make a variable so we can add the side bar stuff to them
		self.sideFrame = sideFrame

		# Create the tabe and frames for the sidebar items
		notebookLabelImages = [gtk.Image(), gtk.Image()]
		notebookLabelImages[0].show()
		notebookLabelImages[1].show()
		notebookLabelImages[0].set_from_file(self.imagePath + "sidebar/systembrowser.png")
		notebookLabelImages[1].set_from_file(self.imagePath + "sidebar/bookmarks.png")

		# Append the images to the note book as well as the frames
		self.mainNotebook.append_page(sideFrame[0], notebookLabelImages[0])
		self.mainNotebook.append_page(sideFrame[1], notebookLabelImages[1])


	################################################################
	# This function creates the bookmarks in the sidebar notebook
	################################################################		
	def makeBookmarkBrowser(self):	

		# Make a model to hold string types
		self.bookmarkModel = gtk.TreeStore(gobject.TYPE_STRING ,gobject.TYPE_STRING)

		# Make a tree view object to hold the model
		self.bookmarkView = gtk.TreeView(self.bookmarkModel)
		self.bookmarkView.show()

		# Make the tree view clickable
		self.bookmarkView.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.bookmarkView.connect("button-press-event", self.openBookmark)

		# Allow the bookmarks browser to receive folders and bookmark them
		targets = [ ( "text/plain", 0, 80 )]
		self.bookmarkView.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,  gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
		self.bookmarkView.selection_add_target("PRIMARY", "COMPOUND_TEXT", 1)
		self.bookmarkView.connect("drag-drop", self.addBookmark)			

		# Make a cell renderer to hold the text
		self.bookmarkRenderer = gtk.CellRendererText()

		# Create a label column, set the header as a save button, then add it to the view
		self.bookmarkColumn = gtk.TreeViewColumn("Click Here to Save", self.bookmarkRenderer, text=1)
		self.bookmarkColumn.set_clickable(gtk.TRUE)
		self.bookmarkColumn.connect("clicked", self.saveBookmarks)	
		self.bookmarkView.append_column(self.bookmarkColumn)

		# Try and read the bookmarks xml file (if it exists)
		bookmarks = self.readBookmarks()

		# Make a variable to hold the iters
		self.bookmarkList = []

		# For each bookmark from the file
		for bookmark in bookmarks:
			# Make an iter
			anIter = self.bookmarkModel.insert_before(None, None)

			# Set the values to the ones from the file
			self.bookmarkModel.set_value(anIter, 0, bookmark[0])
			self.bookmarkModel.set_value(anIter, 1, bookmark[1])

			# Append the bookmark to the list
			self.bookmarkList.append(anIter)

		# Create a scrolled window to hold the tree view, then add 
		# the tree view to it (with the image on the tab)
		self.bookmarkScroll = gtk.ScrolledWindow()
		self.bookmarkScroll.show()
		self.bookmarkScroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.bookmarkScroll.add_with_viewport(self.bookmarkView)
		self.sideFrame[1].add(self.bookmarkScroll)				


	######################################
	# This file reads an existing bookmarks file 
	######################################
	def readBookmarks(self):
		# Set the users storage directory
		saveDir =  os.environ['HOME'] + "/" + ".claw/"

		# Make a variable to hold the bookmarks
		bookmarks = []

		# If a bookmarks file exists ...
		if os.path.exists(saveDir + "bookmarks.xml"):

			# Create a DOM tree from the file
			markFile = minidom.parse(saveDir + "bookmarks.xml")

			# Get the bookmark tags
			listTag = markFile.getElementsByTagName("list")
			bookmarkTags = listTag[0].getElementsByTagName("bookmark")

			# Get the name and locations from the tags
			for tag in bookmarkTags:	
				name = tag.attributes["name"]
				location = tag.childNodes[0].data
				bookmarks.append([location, name.value])

			return bookmarks

		else:
			# If there no file, return some empty values
			return [["", ""]]


	######################################
	# The callback clicking the header (save bookmarks) 
	######################################
	def saveBookmarks(self, button):

		# Set the users storage directory
		saveDir =  os.environ['HOME'] + "/" + ".claw/"

		# First we create a bookmarks file (and directory if one doesn't exist)
		if not os.path.exists(saveDir):
			os.mkdir(saveDir)

		# Open a file to save the bookmarks in	
		bookmarkFile = file(saveDir + "bookmarks.xml", 'w')

		# Add a opening header
		bookmarkFile.write("<!-- This file contains the bookmarks for the CLAW file browser -->")
		bookmarkFile.write("\n<list>")

		# Get the information from the bookmarks list, then add it to the file
		for iter in self.bookmarkList:
			location = self.bookmarkModel.get_value(iter, 0)
			name = self.bookmarkModel.get_value(iter, 1)			
			bookmarkFile.write("\n\t<bookmark name =\"" + name + "\">" + location +"</bookmark>")

		# Add a closing header	
		bookmarkFile.write("\n</list>")

		# Close the file
		bookmarkFile.close()

		# Let the user know what we are up to
		self.infoBar.set_text("Bookmarks Saved")


	######################################
	# The callback for a drop on the bookmarks tree
	######################################
	def addBookmark(self, widget, event, x, y, time):

		# Get the source widget (the event box)
		passed = event.get_source_widget()

		# Get the location of the icon from the current contents	
		location = self.currentContents[self.activeWindow][passed][2]
		name = os.path.basename(location)	

		# If the passed file is a directory, add it and it's location to the tree model
		if os.path.isdir(os.path.abspath(name)):
			anIter = self.bookmarkModel.append(None, None)
			self.bookmarkModel.set_value(anIter, 0, location)
			self.bookmarkModel.set_value(anIter, 1, name)
			self.bookmarkList.append(anIter)
		else:
			# Otherwise, Inform the user that the ...
			self.infoBar.set_text("Bookmark Must Be a Directory")


	######################################
	# The callback for the "clicked" event on the bookmarks tree 
	######################################
	def openBookmark(self, bookmarkView, event):

		# Get the tree selection iter
		treeViewSelection =  bookmarkView.get_selection()

		# get the selected model and iter
		treeStore, selectedIter = treeViewSelection.get_selected()

		# If the iter is None (not on a bookmark) don't do anything
		if selectedIter == None:
			return 0

		# If the bookmark was "doube-clicked" then ...
		if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:

			# If the "Empty" bookmark is clicked, the do nothing
			if treeStore.get_value(selectedIter, 0) == "":
				return 0
			else:
				# Otherwise, set the current directory to the bookmark, then list
				self.currentDirectory[self.activeWindow]= treeStore.get_value(selectedIter, 0)
				self.removeOldIcons()
				self.listCurrentDirectory()

		# If the bookmark is "right clicked"...
		elif event.button == 3:

			# Make a menu
			self.bookmarkMenu = gtk.Menu()
			self.bookmarkMenu.show()

			# Add a REMOVE item to the menu
			removeItem = gtk.MenuItem("Remove")
			removeItem.show()
			removeItem.connect("activate",self.bookmarkMenuClicked, treeStore, selectedIter)
			self.bookmarkMenu.append(removeItem)

			# Make the menu "Popup" immediately
			self.bookmarkMenu.popup(None, None, None, 0, 0)


	######################################
	# The callback for the "activate" event on the popup menu
	######################################
	def bookmarkMenuClicked(self, removeItem, treeStore, selectedIter):

		# Remove the bookmark (i.e destroy it)

		# First we find the bookmark by matching the locations (to be certain)
		for iter in self.bookmarkList:
			if self.bookmarkModel.get_value(selectedIter, 0) == self.bookmarkModel.get_value(iter, 0):

				# Then we remove the iter from the bookmark list and the tree
				self.bookmarkList.remove(iter)
				treeStore.remove(selectedIter)	
				return 0


	################################################################
	# This function creates the quick system browser for the side notebook
	################################################################
	def makeQuickSystemBrowser(self):

		# Create our quick browser model with a picture and one string (name) and a second (location)
		self.mainModel = gtk.TreeStore(gtk.gdk.Pixbuf ,gobject.TYPE_STRING, gobject.TYPE_STRING)

		# Create a tree view object and add the model to it
		self.mainView = gtk.TreeView(self.mainModel)
		self.mainView.show()

		# Create a pixbuf (copy of an image in memory) and scale it to 22x22
		folderPixbuf = gtk.gdk.pixbuf_new_from_file(self.imagePath + "sidebar/browse.png")
		folderPixbuf = folderPixbuf.scale_simple(22, 22, gtk.gdk.INTERP_BILINEAR)

		# Get the base directory listing and sort it alphabetically
		dirList = os.listdir("/")
		dirList.sort(lambda x,y : cmp (string.lower(x), string.lower(y)))

		# Add the picture, the directory names and locations to the model
		for file in dirList:
			if os.path.isdir("/" + file):
				anIter = self.mainModel.insert_before(None, None)
				self.mainModel.set_value(anIter, 0, folderPixbuf)
				self.mainModel.set_value(anIter, 1, file)
				self.mainModel.set_value(anIter, 2, "/" + file + "/")

		# Create an image and text renderer
		self.imageRenderer = gtk.CellRendererPixbuf()
		self.mainRenderer = gtk.CellRendererText()

		# Create a column, add the renderers and then add the column to the main view
		self.mainColumn = gtk.TreeViewColumn("Explore!")
		self.mainColumn.pack_start(self.imageRenderer, gtk.FALSE)
		self.mainColumn.add_attribute(self.imageRenderer, 'pixbuf', 0)
		self.mainColumn.pack_start(self.mainRenderer, gtk.TRUE)
		self.mainColumn.add_attribute(self.mainRenderer, 'text', 1)
		self.mainColumn.set_visible(gtk.TRUE)
		self.mainView.append_column(self.mainColumn)

		# Connect the main view to the button press event signal (mouse clicks)
		self.mainView.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.mainView.connect("button-press-event", self.cellClicked)

		# Make the main view a destination for drags (this allows moving files to remote locations)
		self.mainView.enable_model_drag_dest([("text/plain",0,0)], gtk.gdk.ACTION_DEFAULT)
		self.mainView.connect("drag-drop", self.fileDropped)

		# Create a scrolled window, add the tree view to the window and add the 
		# scrolled window to the the notebook
		self.treeScroll = gtk.ScrolledWindow()
		self.treeScroll.show()
		self.treeScroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.treeScroll.add_with_viewport(self.mainView)
		self.sideFrame[0].add(self.treeScroll)


	######################################
	# The callback for the drag drop on the tree view
	######################################
	def fileDropped(self, passedEventBox, event, x, y, time):

		# First we get the path (using the x,y value)
		path =  passedEventBox.get_dest_row_at_pos(x,y)

		# Use the path to get the iter (location in model)
		iter =  passedEventBox.get_model().get_iter(path[0])

		# Get the location stored in the iter's location and the location of the dropped file
		dropLocation = self.mainModel.get_value(iter, 2) 
		
		# Worthless information, but cool to have       
		dragLocation = self.currentContents[self.layoutOver][event.get_source_widget()][2]

		# See if the selection is an http:// address
		if re.findall("http://", dragLocation) != []:
			self.infoBar.set_text(selection + " is an http:// address!")
			return 0

		# Get the selected icon
		selectedEventBoxes = self.getSelected()

		# Get the locations stored at the locations	
		selected = []
		for eventBox in selectedEventBoxes:
			selected.append(self.currentContents[self.activeWindow][eventBox][2]) 

		# If you are trying to drag a directory on top of itself, remove that entry from the list	
		for name in selected:
			if name == dropLocation:
				self.infoBar.set_text(os.path.basename(location) + " is the same file as " + os.path.basename(name))
				selected.remove(name)


		# See if the passed file(s) exist(s) in the destination directory
		directoryList = os.listdir(dropLocation)
		for icon in selected:
			if os.path.basename(name) in directoryList:

				# If there is another file with the same name, see if they want it overwritten
				answer = overwrite.getAnswer(name, self)
				if answer == "no":
					selected.remove(icon)
				# If they want it overwritten, then remove the old directory or file
				else:
					if os.path.isdir(dropLocation + os.path.basename(name)):
						self.deleteFile(dropLocation + os.path.basename(name))
					elif os.path.isfile(dropLocation + os.path.basename(name)):
						self.deleteFile(dropLocation + os.path.basename(name))

		# If no files survived the screening, then quit
		if len(selected) == 0:
			self.infoBar.set_text("Cannot Move Files to : " + dropLocation)
			return 1

		# Create a popup menu for the drop (if you can drop in the selected directory)
		if os.access(dropLocation, os.W_OK):

			# Create a popup menu
			self.quickMenu = gtk.Menu()
			self.quickMenu.show()

			# Add an COPY item to the menu and connect it to a callback
			copyItem = gtk.MenuItem("Copy Here")
			copyItem.show()
			copyItem.connect("activate", self.quickDrop, dropLocation, selected, "Copy")
			self.quickMenu.append(copyItem)

			# Add a MOVE item to the menu and connect it to a callback
			moveItem = gtk.MenuItem("Move Here")
			moveItem.show()
			moveItem.connect("activate", self.quickDrop, dropLocation, selected, "Move")
			self.quickMenu.append(moveItem)

			# Add a LINK item to the menu and connect it to a callback
			linkItem = gtk.MenuItem("Link Here")
			linkItem.show()
			linkItem.connect("activate", self.quickDrop, dropLocation, selected, "Link")
			self.quickMenu.append(linkItem)

			# make the menu a popup (which immediately visualizes)
			self.quickMenu.popup(None, None, None, 0, 0)

		else:
			self.infoBar.set_text("Cannot drop into " + dropLocation + " : Check Permissions")


	######################################
	# The callback for the quick browser popup menu 
	######################################		
	def quickDrop(self, label, dropLocation, selected, droptype):

		# Go through the selected files
		for begin in selected:

			# Make a drop point
			end = dropLocation + "/" + os.path.basename(begin)	

			# Let's check the type of drops
			if droptype == "Copy":
				try:
					# Let the user know what is happening
					self.infoBar.set_text("Copying " + os.path.basename(begin) + " to " + end)

					# Use the progress dialog to copy the files
					pg = progressdialog.fileMover(self,  begin, end)

				except:
					# Let the user know of any errors
					self.infoBar.set_text("Cannot Copy " + os.path.basename(begin)  + " to " + end)

			elif droptype == "Link":
				try:
					self.infoBar.set_text("Created Link from " + os.path.basename(begin) + " to " + end)
					os.symlink(begin, end)
				except:
					self.infoBar.set_text("Cannot Create Link " + os.path.basename(begin)  + " to " + end)

			elif droptype == "Move":
				try:
					self.infoBar.set_text("Copying " + os.path.basename(begin) + " to " + end)
					
					# Use the progress dialog to copy the files
					pg = progressdialog.fileMover(self,  begin, end)
						
					# Erase the moved file
					if os.path.isdir(begin):
						shutil.rmtree(begin)
					else:
						os.remove(begin)

				except:
					self.infoBar.set_text("Cannot Move " + os.path.basename(begin) + " to " + end)


	######################################
	# The callback for the button press event on the tree view 
	######################################		
	def cellClicked(self, treeView, event):

		# Get the selected tree model from the tree view (which is passed as widget)
		treeViewSelection = treeView.get_selection()
		# Get the tree model and the iter that were selected
		treeStore, selectedIter = treeViewSelection.get_selected()

		# If the right button was clicked and the iter isn't blank (clicked on nothing)...
		if event.button == 3 and selectedIter != None:

			# Create a popup menu		
			self.shortcutMenu = gtk.Menu()
			self.shortcutMenu.show()

			# Add an OPEN item to the menu and connect it to a callback
			openItem = gtk.MenuItem("Open")
			openItem.show()
			openItem.connect("activate", self.shortcutClicked, "Open", selectedIter, treeStore)
			self.shortcutMenu.append(openItem)

			# make the menu a popup (which immediately visualizes)
			self.shortcutMenu.popup(None, None, None, 0, 0)

		# If the left button was "double-clicked" then ...
		elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
			
			# If there was a row selected (i.e. not None) ... 
			if selectedIter != None:

				# Erase any existing clildren (existing directory listings)
				# of the clicked selection (the parent directory)
				while self.mainModel.iter_has_child(selectedIter):
					anIter = self.mainModel.iter_children(selectedIter)
					if anIter != None:
						self.mainModel.remove(anIter)

				# Get the "location" info we put in the selected iter
				directory = treeStore.get_value(selectedIter, 2)

				# Check permissions
				if os.access(directory, os.R_OK):

					# Add children to the selected iter (directory on the tree)
					# based on the folders inside the parent folder
					self.expandDir(directory , selectedIter)				
				else:
					self.infoBar.set_text("You don't have permission to read this directory")


	######################################
	# The callback for when the OPEN is selected from the right click menu 
	######################################
	def shortcutClicked(self, menuItem, event, selectedIter, treeStore):

		# Get the "location" value we put into the iter
		directory = self.mainModel.get_value(selectedIter, 2)	

		# Clear the current directory of icons
		self.removeOldIcons()

		# Set the current directory for the currently selected icon view to the selected location
		self.currentDirectory[self.activeWindow]= os.path.dirname(directory)

		# List the contents of the directory and put them in the currenty selected icon view
		self.listCurrentDirectory()	


	######################################
	# The callback for when the OPEN is selected from the right click menu
	######################################
	def expandDir(self, path, parentIter):

		# Get the contents of the passed directory
		dirList = os.listdir(path)
		dirList.sort(lambda x,y : cmp (string.lower(x), string.lower(y)))

		# Create a pixbuf (copy in memory) of an image (png) from a file and scale to 22x22
		folderPixbuf = gtk.gdk.pixbuf_new_from_file(self.imagePath + "sidebar/browse.png")
		folderPixbuf = folderPixbuf.scale_simple(22, 22, gtk.gdk.INTERP_BILINEAR)

		# For each directory in the current directory list, 
		# Add icon, a title and a location (real path)	
		for file in dirList:
			if os.path.isdir(path + file):
				anIter = self.mainModel.append(parentIter)
				self.mainModel.set_value(anIter, 0, folderPixbuf)
				self.mainModel.set_value(anIter, 1, file)
				self.mainModel.set_value(anIter, 2, path + file + "/")

		# Expand the row (show all the children)
		self.mainView.expand_row(self.mainModel.get_path(parentIter), gtk.FALSE)


	##############################################################
	# This controls the full directory listing sequence
	##############################################################		
	def listCurrentDirectory(self, widget=None, event=None):

		# Let's add a timing element (for our own sake)
		print "\n++++++++++++++++++++++++++++"
		print "+ Start Listing Directory at:" , time.strftime("%H:%M:%S", time.localtime())
		starttime = time.clock()
		
		# Use the visible "waiting" dialog to show that user we are working
		self.progressBar = waiting.waitingDialog("Loading Directory", self)

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
				gtk.main_iteration()	

		# Listing process is started
		self.listing = 1
		
		# We are not over any icons
		self.over = 0

		# Add the current directory to the history list (keep it to 30)
		if self.backing == 0:
			self.history[self.activeWindow].append(self.currentDirectory[self.activeWindow])
			if len(self.history[self.activeWindow]) > 30:
				self.history[self.activeWindow].remove(0)
		else:
			self.backing = 0
	
		# Make labels for the icons
		self.progressBar.set_text("Creating Labels")
		self.progressBar.set_fraction(0.10)

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
				gtk.main_iteration()	

		self.makeLabels()
		nexttime = time.clock()
		print "+ Labels Created :", nexttime - starttime

		# Make icons based on filetype
		self.progressBar.set_text("Creating Icon Images")
		self.progressBar.set_fraction(0.20)

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
				gtk.main_iteration()	
				
		# Pass the progress bar to this function since it takes
		# far longer than the others
		self.makeIcons()
		nexttime = time.clock() - nexttime
		print "+ Icon Images Created :", nexttime

		# Format the icons and labels for display
		self.progressBar.set_text("Formatting Icons")	
		self.progressBar.set_fraction(0.80)	

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
				gtk.main_iteration()	

		self.formatIcons(self.progressBar)
		nexttime = time.clock() - nexttime
		print "+ Icons Formatted :", nexttime
		
		# Place the icons in the layout
		self.progressBar.set_text("Placing Icons in Layout")
		self.progressBar.set_fraction(0.90)

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
				gtk.main_iteration()	

		self.placeIcons()
		nexttime = time.clock() - nexttime
		print "+ Icons Places in Window :", nexttime
		
		# Move to the current directory (this makes working in it easier)
		os.chdir(self.currentDirectory[self.activeWindow])

		# See if the user wants the current address listed in the address bar
		if self.clearAddress == 0:
			self.mainCombo.set_popdown_strings([self.currentDirectory[self.activeWindow]])
		# If not, then clear it
		else:
			self.mainCombo.set_popdown_strings([""])

		# Close the progress bar
		self.progressBar.set_fraction(1.0)
		waiting.quit()
		
		print "+ Directory Open Process Complete At:", time.strftime("%H:%M:%S", time.localtime())
		endtime = time.clock()
		fileNumber = len(os.listdir(self.currentDirectory[self.activeWindow])) 
		print "+", fileNumber, "Files"
		print "+ Total Time :", int(endtime) - int(starttime),"seconds"
		print "++++++++++++++++++++++++++++\n"
		
		# Create an instance of the notify class, it will keep track of files changes not by the Claw3
		self.notifier[self.activeWindow] = notify.DirectoryNotifier(self.currentDirectory[self.activeWindow], self)
		signal.signal(signal.SIGIO, lambda sig, frame: self.notifier[self.activeWindow](frame))
		
		# Okay, we're done listing
		self.listing = 0


	##############################################################
	# This creates icons based on the files in a directory
	##############################################################
	def makeLabels(self):
		
		# Create a list to hold the labels
		currentLabels = []
		
		# This makes things faster
		font =self.font
		currentDirectory = self.currentDirectory
		activeWindow = self.activeWindow

		# List the items in the current directory and add them to a list
		contents = os.listdir(currentDirectory[activeWindow])

		# Filter out the hidden files and folders
		contents =  filter(lambda x : x[0] != '.', contents)	

		# Sort the list alphabetically
		contents.sort(lambda x,y : cmp (string.lower(x), string.lower(y)))

		# Filter out the folders
		folders =  filter(lambda x : os.path.isdir(currentDirectory[activeWindow]+ "/" + x), contents)		

		# Place the folders at the beginning of the list in the same order they were removed
		for z in range(len(folders)):
			contents.remove(folders[z])
			contents.insert(z, folders[z])
			
		# Set a variable to hold the directory contents' locations
		locations = []
		for line in contents:
			if currentDirectory[activeWindow]!= "/":
				locations.append(currentDirectory[activeWindow]+ "/"+ line)
			else:
				locations.append(currentDirectory[activeWindow]+ line)

		# Create a label widget and add it to a list
		for item in contents:
			
			# Get the size of the font and use it to get the textwrap length
			try:
				size = font.get_size()/1000
			except  AttributeError:
				size = font.getsize()/1000				
			wrap = 26 - size

			item = textwrap.fill(item, wrap)

			# Create and format the label
			label = gtk.Label(item)
			label.modify_font(font)
			label.set_justify(gtk.JUSTIFY_CENTER)
			label.set_line_wrap(gtk.TRUE)
			label.show()
			currentLabels.append(label)

		# Make global versions of these locals
		self.locations = locations
		self.currentLabels = currentLabels


	##############################################################
	# This creates icons based on the files in a directory
	##############################################################
	def makeIcons(self):
		
		# Supposedly, using a local is faster than a global
		locations = self.locations
		currentLabels = self.currentLabels

		# Use the built in python to get the mime types
		types = map(lambda location : mimetypes.guess_type(location)[0], locations)

		# We create an image per file (imagine that)
		icons = []
		for x in range(len(locations)):
			icons.append(gtk.Image())	

		# Create a dictionary, locations to types			
		contents = dict(zip(locations, zip(types, currentLabels, icons)))	
		
		# Clear the variables
		icons, types, currentLabels = [],[],[]
		
		for location in locations:
	
			# Make a list from the contents in contents
			contents[location] = list(contents[location])
			
			try:
				contents[location][0] = string.split(contents[location][0], "/")
			except:
				contents[location][0] = "None"
				
		# Find the mime types for the icons
		contents = self.sort_icons(contents)
		
		# Create icons from the mimetypes
		self.create_icons(contents)
		
	#########################################	
	# This function sorts the icons according to mime type
	#########################################
	def sort_icons(self, contents):
	
		# This should make things faster
		locations = self.locations
		mountedpoints = self.mountedpoints
		mountpoints = self.mountpoints
		
		self.progressBar.set_text("Getting Mime Types")		
		
		# Sort out the folders
		folders = filter(os.path.isdir, locations)
		if folders != []:
			for location in folders:
				contents[location][0] = ["folder", "folder"]
		
		# Sort out the mounted partitions
		mounted = filter(os.path.ismount, locations)
		if mounted != []:
			for location in mounted:
				if re.findall("cdrom", location):
					group, mime = "mounted","cdrom_mounted"
				elif re.findall("dvd", location):
					group, mime =  "mounted","dvd_mounted"
				else:
					group, mime =  "mounted","harddrive_mounted"		
				
				contents[location][0] = [group, mime]
	
		# Sort out the unmounted partitions
		unmounted = filter(lambda location: location in mountpoints and location not in mounted, locations)	
		if unmounted != []:
			for location in unmounted:
				if re.findall("cdrom",	location):
					group, mime = "unmounted","cdrom_unmounted"
				elif re.findall("dvd", location):
					group, mime = "unmounted","dvd_unmounted"
				else:
					group, mime = "unmounted","harddrive_unmounted"	
				
				contents[location][0] = [group, mime]

		# Set the progressBar amount
		self.progressBar.set_fraction(0.30)
			
		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()
	
		# Sort out unrecognized filetypes and use the file command
		nonetypes = filter(lambda location: contents[location][0] == "None" or contents[location][0][1] == "plain", contents)

		if nonetypes != []:
			types =	[commands.getoutput("file -ibL " + self.returnClean(location)) for location in nonetypes]
			types = [re.split("[;,]",filetype)[0] for filetype in types]
		
			for x in range(len(nonetypes)):
				
				# See if the file command returned anything useful
				try:	
					group, mime = string.split(types[x], "/")
				except:
					group, mime = "text","plain"
			
				# If the file command can't guess it, then it is unknown or we don't have permission
				if group not in ["application","audio","image","text","video"]:
					 group, mime = "text","plain"
					
				contents[nonetypes[x]][0] = [group, mime]	

		# Set the progressBar amount
		self.progressBar.set_fraction(0.30)
			
		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()
					
		return contents

	#########################################	
	# This function create icons according to mimetype
	#########################################					
	def create_icons(self, contents):
	
		imagePath = self.imagePath
		self.progressBar.set_text("Creating Icons")	
	
		# Sort out the images and make previews
		images = filter(lambda location: contents[location][0][0] == "image" and  contents[location][0][1] not in ["gimp", "x-3ds"],contents)
		
		if images != []:
			for location in images:

				# Get some useful information from the contents
				group, mime = contents[location][0][0], contents[location][0][1]

				# Try to create a pixbuf
				try:
					iconPixbuf = gtk.gdk.pixbuf_new_from_file(location)	
					# Scale the pixbuf to the right size (48x48)
					iconPixbuf = iconPixbuf.scale_simple(48, 48, gtk.gdk.INTERP_BILINEAR)				
		
				# If the file can't be read for some reason, then use a default icon
				except:
					iconPixbuf = gtk.gdk.pixbuf_new_from_file(imagePath + "icon/plain.png")
			
				# Set the image
				contents[location][2].set_from_pixbuf(iconPixbuf)

		# Set the progressBar amount
		self.progressBar.set_fraction(0.40)
			
		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()			
			
		# Sort out everything that isn't an image and create the icons
		not_images = filter(lambda location :location not in images, contents)

		if not_images != []:
			for location in not_images:

				# Get some useful information from the contents
				group, mime = contents[location][0][0], contents[location][0][1]
				
				# Try to create a pixbuf
				try:
					contents[location][2].set_from_pixbuf(self.pixbufs[mime+".png"])
		
				# If the file can't be read for some reason, then use a default icon
				except:
					iconPixbuf = gtk.gdk.pixbuf_new_from_file(imagePath + "icon/plain.png")
					contents[location][2].set_from_pixbuf(iconPixbuf)

							
		# Check for links and show the icons
		links = filter(os.path.islink, contents)
		if links != []:
			for location in links:

				# If this is a link, make it known
				if os.path.islink(location):
					contents[location][1].set_text(contents[location][1].get_text() + "\n(Link)")
					contents[location].append("link")


	
		# Update the progressBar amount
		self.progressBar.set_fraction(0.60)
		while  gtk.events_pending() :
			gtk.main_iteration()
		
		# Make global copies for later use
		self.contents = contents

	##############################################################
	# Format the icons
	##############################################################
	def formatIcons(self, progressBar):

		# Since locals are faster than globals, let's create some local versions of globals
		font = self.font
		fontColor = self.fontColor
		locations = self.locations
		layoutColor = self.layoutColor
		contents = self.contents
		showPermissions = self.showPermissions

		# Get the fraction for the progress bar (beware folders with no content)
		if len(locations) == 0:
			fraction = .1
		else:
			# Divide .1 by the number of things in the directory
			fraction = .1/len(locations)
		fractions = 0

		# Create/Clear a dictionary to hold the file information
		self.currentContents[self.activeWindow] = {}

		# This will be used for listing (otherwise you can't keep them in order)
		self.eventBoxes[self.activeWindow] = []

		# Add the icons to the window
		for location in locations:
		
			# Create a box to hold the label and icon
			box = gtk.VBox(gtk.FALSE, 2)

			# Create an event box to capture events for the icon
			eventBox = gtk.EventBox()
			eventBox.set_size_request(60, 60)

			# Add the icon to the eventbox and add it event box to the main box
			eventBox.add(contents[location][2])
			box.pack_start(eventBox, gtk.FALSE, gtk.TRUE)

			# Set the chosen font and color for the text in the label and the box
			contents[location][1].modify_fg(gtk.STATE_NORMAL, fontColor)
			contents[location][1].modify_font(font)
			eventBox.modify_bg(gtk.STATE_NORMAL, layoutColor)

			# Add the label to the box
			box.pack_start(contents[location][1], gtk.FALSE, gtk.TRUE)

			# Add the information labels if desired
			if showPermissions == 'yes':
				fileSize, permissions, user = self.getFileInfo(location)
				labels = [gtk.Label(user), gtk.Label(fileSize), gtk.Label(permissions)]
				labels[0].modify_font(font)
				labels[0].modify_fg(gtk.STATE_NORMAL, fontColor)
				labels[1].modify_font(font)
				labels[1].modify_fg(gtk.STATE_NORMAL, fontColor)
				labels[2].modify_font(font)
				labels[2].modify_fg(gtk.STATE_NORMAL, fontColor)
				box.pack_start(labels[0], gtk.FALSE, gtk.TRUE)
				box.pack_start(labels[1], gtk.FALSE, gtk.TRUE)
				box.pack_start(labels[2], gtk.FALSE, gtk.TRUE)

			# Make the eventbox and all stuff inside visible

			# Wait until the event is finished before continuing
			box.show_all()
			eventBox.show()

			# These allow the icon to capture events, like button presses and state changes
			eventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
			eventBox.connect("button-press-event", self.iconClicked)
			eventBox.connect("state-changed", self.iconStateChanged)

			# These are to capture mouse over icon events w/o the layout capturing them
			self.over = 0
			eventBox.connect("enter-notify-event", self.overIcon)
			eventBox.connect("leave-notify-event", self.notOverIcon)

			# Connect the event box to drag and drop events
			targets = [("text/plain", 0, 0)]
			eventBox.drag_source_set(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
			eventBox.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,  gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)

			# This is activated when something with information drops on the receiving widget
			eventBox.connect('drag_data_get', self.dataGet)
	
			# This is activated when the receiving widget has gotten the information
			eventBox.connect("drag-data-received", self.iconDropped)
			
			# Make sure to use the proper icon when dragging (so it looks like we are dragging)
			eventBox.drag_source_set_icon_pixbuf(contents[location][2].get_pixbuf())
		
			# Add the event box to the list in order
			self.eventBoxes[self.activeWindow].append(eventBox)

			# Add the event box to the contents list	
			self.currentContents[self.activeWindow][eventBox] = [contents[location][2], contents[location][1], location, contents[location][0], box]

		# Set the progressBar amount
		progressBar.set_fraction(0.80 + fractions)
		fractions += fraction
			
		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()

	# Testing this
	def dropped(self, widget, drag_context):
		print "Dropped"


	##############################################################
	# This adds the icons to the layout 
	##############################################################
	def placeIcons(self, progressBar=None):

		# This prevents partial loading
		activeWindow = self.activeWindow
		eventBoxes = self.eventBoxes
		mainLayout = self.mainLayout

		# Get the fraction for the progress bar (beware folders with no content)
		if len(self.locations) == 0:
			fraction = .1
		else:
			# Divide the .10 we have to move by the
			# number of files
			fraction = .1/len(self.locations)
		fractions = 0

		# This just makes things faster
		currentContents = self.currentContents

		# Set the original x andy positions to 0
		x, y, height = 30, 15, 0 
		self.width[activeWindow] = 0

		# Get the size of the layout window
		width = self.mainLayout[activeWindow].allocation.width

		# This is for startup
		if width == 1:
			width = 480

		#iconsPerRow = width/150
		#print zip(*[iter(eventBoxes[activeWindow])]*3)
		
		# This will help with formatting the y direction
		rowicons = []

		# Add the event boxes to the layout
		for box in eventBoxes[activeWindow]:

			mainLayout[activeWindow].put(currentContents[activeWindow][box][4], x - currentContents[activeWindow][box][4].allocation.width, y)
			rowicons.append(currentContents[activeWindow][box][4])	

			# Make sure that the icon spacing is right
			x += 140			
			
			# Wait until the box is added so we can get accurate measurements
			while  gtk.events_pending() :
				gtk.main_iteration()		

			# If we have reached maximum width reset the values and add to the height
			if x + 130 > width:

				height = max(map(lambda icon : icon.allocation.height, rowicons))
				[icon.set_size_request(140, height) for icon in rowicons]
				
				y+= height + 20

				height = 0
				rowicons = []

				x=30
		
			if x > self.width[activeWindow]:
				self.width[activeWindow] = x

								
		# Set the progressBar amount
		if progressBar !=None:
			progressBar.set_fraction(0.90)
			
		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()	
		
		# Set the last row correctly
		if rowicons != []:
			height = max(map(lambda icon : icon.allocation.height, rowicons))
			[icon.set_size_request(140, height) for icon in rowicons]

		self.mainLayout[activeWindow].show_all()

		# Set the size of the layout, so the scroll bars know what to do
		self.mainLayout[activeWindow].set_size(self.width[activeWindow] + 130, y + height)

		# Wait until the event is finished before continuing
		while  gtk.events_pending() :
			gtk.main_iteration()	
				
		# Also, set the window size (Set this as option)
		#if y < gtk.gdk.screen_height() - 300 and self.width > 415:
		#	self.mainWindow.resize(self.mainWindow.get_size()[0], y + 80)
		#elif y < gtk.gdk.screen_height() - 300  and self.width <= 415:
		#	self.mainWindow.resize(415, y + 80)		
		#else:
		#	self.mainWindow.resize(self.mainWindow.get_size()[0], gtk.gdk.screen_height() -300)

		
		# Clear the variables
		self.currentIcons = []
		self.currentLabels = []
		self.locations = []
		self.contents = []
		
		# We're done listing
		self.listing = 0


	##############################################################
	# This checks the size of the window and calls for reloading if necessary
	##############################################################
	def resizeLayout(self, layout, event):
	
		# Make sure we aren't listing right now
		if self.listing == 0:
		
			# Check to see if the window is too wide
			if self.mainLayout[self.activeWindow].get_allocation().width >= self.width[self.activeWindow] + 300  or self.mainLayout[self.activeWindow].get_allocation().width < self.width[self.activeWindow] and self.width[self.activeWindow] > 180:
	
				# Set the listing
				self.listing = 1
			
				# Remove all the old icons from the layout
				self.removeOldIcons()
			
				# Reload the icons
				self.placeIcons()


	##############################################################
	# This removes old icons from the layout so new ones can go in
	##############################################################		
	def removeOldIcons(self):

		# We are listing, so let resize know
		self.listing = 1	

		# This prevents partial erasure
		activeWindow = self.activeWindow

		# Remove all the old icons from the layout
		for icon in self.currentContents[activeWindow]:
			self.mainLayout[activeWindow].remove(self.currentContents[activeWindow][icon][4])



	##############################################################
	# This function handles signals that the current directory has changed
	##############################################################			
	def contentsChanged(self, filename, event):

		# Use reload to handle erased files	
		if event == "removed":
			self.goReload()
			
		# Relist if a file has been added
		elif event == "added":
			self.removeOldIcons()
			self.listCurrentDirectory()


	##############################################################
	# MOUSE INTERACTION SECTION
	##############################################################

	##############################################################
	# This is the callback when an icon is clicked
	##############################################################
	def iconClicked(self, eventBox, event):

		# Set the active window to the window with the event box in it
		if eventBox in self.currentContents[0]:
			self.activeWindow = 0
		else:
			self.activeWindow = 1

		# First of all, what kind of click was it
		if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
	
			# Check to see if a directory was clicked
			if os.path.isdir(os.path.abspath(self.currentContents[self.activeWindow][eventBox][2])):
			
				# Check to see if you haver permissions to enter the directory
				if os.access(os.path.abspath(self.currentContents[self.activeWindow][eventBox][2]), os.R_OK):

					# Change the current directory (the one we list)
					self.currentDirectory[self.layoutOver]= os.path.abspath(self.currentContents[self.activeWindow][eventBox][2])
					
					# Remove all the old icons from the layout
					self.removeOldIcons()
					
					# List the new directory
					self.listCurrentDirectory()
				else:

					# Print an error message if your permission are lacking
					self.infoBar.set_text("You don't have permissions to enter " + os.path.basename(self.currentContents[self.layoutOver][eventBox][2]))
			else:

				# If this isn't a directory, the let's try and open it with the available apps
				self.executeCommand(self.currentContents[self.layoutOver][eventBox][3], self.currentContents[self.layoutOver][eventBox][2])


		# In case of a single left click, just set the state (a callback will catch this)					
		elif event.button == 1:
			if eventBox.state == gtk.STATE_SELECTED:
				eventBox.set_state(gtk.STATE_NORMAL)	
			else:
				eventBox.set_state(gtk.STATE_SELECTED)

		# In case of a right click		
		elif event.button == 3:
			self.rightClickMenu(event, eventBox)


	######################################
	# This function handles the right click menu
	######################################
	def rightClickMenu(self, event, eventBox):

		# Get our location
		location = self.currentContents[self.activeWindow][eventBox][2]

		# Create a menu
		menu = gtk.Menu()
		menu.show()

		# If this is a dead link, then ignore it except for delete
		if not os.path.exists(location):
			deleteItem = gtk.MenuItem(self.messages["delete"][0])
			deleteItem.show()
			deleteItem.connect("activate", self.menuClicked, "Delete", eventBox)
			menu.append(deleteItem)	

		# If this is a mount point, if it is mounted, then give an unmount selection
		elif self.currentContents[self.activeWindow][eventBox][3][0] == "mounted":
			unmountItem = gtk.MenuItem(self.messages["unmount"][0])
			unmountItem.show()
			unmountItem.connect("activate", self.menuClicked, "Unmount", eventBox)
			menu.append(unmountItem)	

		# If this is an unmounted partition, make a selection to mount it
		elif self.currentContents[self.activeWindow][eventBox][3][0] == "unmounted":
			mountItem = gtk.MenuItem(self.messages["mount"][0])
			mountItem.show()
			mountItem.connect("activate", self.menuClicked, "Mount", eventBox)
			menu.append(mountItem)

			# Also make a selection to eject the disk if this is a cd or dvd drive
			if re.findall("cdrom", os.path.basename(location)) != [] or re.findall("dvd", os.path.basename(location)) != []:
				ejectItem = gtk.MenuItem(self.messages["eject"][0])
				ejectItem.show()
				ejectItem.connect("activate", self.menuClicked, "Eject", eventBox)
				menu.append(ejectItem)

		# If this is nothing else, then it is a regular file
		else:

			# Populate main menu (does this really need explanation?)		
			openItem = gtk.MenuItem(self.messages["open"][0])
			openItem.show()
			openItem.connect("activate",self.menuClicked, "Open", eventBox)
			menu.append(openItem)

			# Don't need an open with for a directory
			if not os.path.isdir(location):
				# Create a sub menu			
				openWithMenu =gtk.Menu()
				openWithMenu.show()

				# Create an open with item
				openWithItem = gtk.MenuItem(self.messages["open_with"][0])
				openWithItem.show()
				# Set this is a sub menu of the menu
				openWithItem.set_submenu(openWithMenu)
				menu.append(openWithItem)

				# If this is a file, we need the list of programs associated with the extention
				apps = mimereader.getApplications(self.currentContents[self.activeWindow][eventBox][3])	
	
				# If there are associated apps, then add them to the menu
				if apps != []:
					for app in apps:
						openWith = gtk.MenuItem(app)
						openWith.show()
						openWith.connect("activate", self.menuClicked, app, eventBox)
						openWithMenu.append(openWith)

				# Then add the "Other" item so we can open this with other apps
				otherItem = gtk.MenuItem(self.messages["other"][0])
				otherItem.show()
				otherItem.connect("activate", self.menuClicked, "Other", eventBox)
				openWithMenu.append(otherItem)

			# This is for separating menu item groupings
			separator =gtk.SeparatorMenuItem()
			separator.show()
			menu.append(separator)

			# Create some manipulation items
			if os.access(location, os.R_OK):
				copyItem = gtk.MenuItem(self.messages["copy"][0])
				copyItem.show()
				copyItem.connect("activate", self.menuClicked, "Copy", eventBox)
				menu.append(copyItem)

			if os.access(location, os.W_OK):
				cutItem = gtk.MenuItem(self.messages["cut"][0])
				cutItem.show()
				cutItem.connect("activate", self.menuClicked, "Cut", eventBox)
				menu.append(cutItem)

			# This is for separating menu item groupings
			separator =gtk.SeparatorMenuItem()
			separator.show()
			menu.append(separator)

			if os.access(location, os.W_OK):
				renameItem = gtk.MenuItem(self.messages["rename_and_premissions"][0])
				renameItem.show()
				renameItem.connect("activate", self.menuClicked, "Rename", eventBox)
				menu.append(renameItem)

			# People like this one (I use it alot myself)
			if os.access(location, os.R_OK):
				archiveItem = gtk.MenuItem(self.messages["create_archive"][0])
				archiveItem.show()
				archiveItem.connect("activate", self.menuClicked, "Create Archive", eventBox)
				menu.append(archiveItem)	

			# This is for separating menu item groupings
			separator =gtk.SeparatorMenuItem()
			separator.show()
			menu.append(separator)

			if not os.path.isdir(location):
				mimeItem = gtk.MenuItem(self.messages["associtate_with_application"][0])
				mimeItem.show()
				mimeItem.connect("activate", self.menuClicked, "Associate With Application", eventBox )
				menu.append(mimeItem)	

				# This is for separating menu item groupings
				separator =gtk.SeparatorMenuItem()
				separator.show()
				menu.append(separator)

			if os.access(location, os.W_OK):
				trashItem = gtk.MenuItem(self.messages["send_to_wastebasket"][0])
				trashItem.show()		
				trashItem.connect("activate",self.menuClicked, "Trash", eventBox)
				menu.append(trashItem)	

				deleteItem = gtk.MenuItem(self.messages["delete"][0])
				deleteItem.show()		
				deleteItem.connect("activate",self.menuClicked, "Delete", eventBox)
				menu.append(deleteItem)

		# Make the menu a "popup"
		menu.popup(None, None, None, event.button, event.time)


	######################################
	# This handles the callbacks for the right click file menu
	######################################
	def menuClicked(self, menubutton, name, eventBox):

		# Get the current location and make it BASH ready
		location = self.currentContents[self.activeWindow][eventBox][2]
		bashlocation = self.returnClean(location)

		# Let's find out what was selected

		# Let's try and open this file or folder
		if name == "Open":
	
			# If it's a directory, then enter it
			if os.path.isdir(location):
				self.currentDirectory[self.activeWindow]= location
				self.removeOldIcons()
				self.listCurrentDirectory()
			else:
				# Otherwise, use the execute function 
				self.executeCommand(self.currentContents[self.activeWindow][eventBox][3], bashlocation)

		# Add some selections to the clipboard
		elif name == "Copy":

			# Get the selected icons
			selected = self.getSelected()

			# Make sure icons were selected
			if selected == []:
				self.infoBar.set_text("Please Select Icons to Copy")
				return 0
			
			# Get the selected icons locations
			locations = map(lambda x : self.currentContents[self.activeWindow][x][2], selected)

			# Open the "Clipboard" (can't wait for pygtk 2.4 stable)
			clipfile = file(os.environ['HOME'] + "/.claw/prettyclipboard.txt", 'w')
			
			# Add the selections to the clipboard
			clipfile.write("copy\n")
			for location in locations:
				clipfile.write(location + "\n")

			# Then close the file
			clipfile.close()

		# Add some selections to the clipboard
		elif name == "Cut":

			# Get the selected icons
			selected = self.getSelected()

			# Make sure icons were selected
			if selected == []:
				self.infoBar.set_text("Please Select Icons to Cut")
				return 0
	
			# Get the selected icons locations
			locations = map(lambda x : self.currentContents[self.activeWindow][x][2], selected)		

			# Open the "Clipboard" (can't wait for pygtk 2.4 stable)
			clipfile = file(os.environ['HOME'] + "/.claw/prettyclipboard.txt", 'w')

			# Add the selections to the clipboard
			clipfile.write("cut\n")
			for location in locations:
				clipfile.write(location + "\n")

			# Then close the file
			clipfile.close()

		# Let's rename some files
		elif name == "Rename":
			
			# Let's get the selected icons
			selected = self.getSelected()		

			# Let's make sure something was actually selected
			if selected == []:
				self.infoBar.set_text("Select Some Files to Rename First")
				return 0

			# First we need the locations
			names = []
			for selection in selected:
				names.append(self.currentContents[self.activeWindow][selection][2])	

			# Lastly, let's check permissions
			for name in names:
				if not os.access(name, os.W_OK):
					names.remove(name)
					self.infoBar.set_text("You don't have permission to rename " + os.path.basename(name))

			rename.Rename(self, names)			

		# Let's create an archive
		elif name == "Create Archive":

			# Make sure we can write to the current directory before creating the archive 
			if os.access(self.currentDirectory[self.activeWindow], os.W_OK):
				try:
					# First we tar the archive if this isn't a zip
					if self.archiver == "zip -r":
						os.popen(self.archiver + " " + os.path.basename(bashlocation) + ".zip" + " " + os.path.basename(bashlocation))
					else:
						# Otherwise we tar it first, then compress it
						os.popen("tar -cvf " + os.path.basename(bashlocation) + ".tar " + os.path.basename(bashlocation))
						os.popen( self.archiver + " "  + os.path.basename(bashlocation) + ".tar")
					
					self.infoBar.set_text("Archiving Succeeded")

				except:
					self.infoBar.set_text("Archiving failed")	

			# If you don't have write permissions, then tar it to your home directory
			else:
				try:
					# First we make a copy to the home directory
					shutil.copy(os.path.basename(location), os.environ['HOME'])

					# Next we move to the home directory
					os.chdir(os.environ['HOME'])

					# Then we tar the archive if this isn't a zip
					if self.archive == "zip":
						os.popen(self.archiver + " " + os.path.basename(bashlocation) + ".zip" + " " + os.path.basename(bashlocation))
					else:
						# Otherwise we tar it first, then compress it
						os.popen("tar -cvf " + os.path.basename(bashlocation) + ".tar " + os.path.basename(bashlocation))
						os.popen(self.archiver + " "  + os.path.basename(bashlocation) + ".tar")

					# Lastly we return to the directory we were in before all this and tell the user what happened
					os.chdir(self.currentDirectory[self.activeWindow])
					self.infoBar.set_text("Archive in located in your HOME directory")

				except:
					# If archiving failed, let the user know
					self.infoBar.set_text("Archiving failed")

		# This allows you to change the applications you open a file with
		elif name == "Associate With Application":

			# We are using the mimereader editor for this
			mimereader.editor(self, eventBox)
			return 0

		#Let's move some files to the trash
		elif name == "Trash":

			# If this is a link, we can't toss it, it must be deleted
			if os.path.islink(location):
				self.infoBar.set_text("You can't put links in the trash. You must delete it.")
				return 1
			
			# We'll use the progress dialog to show what is happening
			progressdialog.fileMover(self, location, os.environ['HOME'] + "/..Wastebasket/" + os.path.basename(location))

			# Then we erase the initial file (I hope)
			if os.access(location, os.W_OK):
				try:
				
					# Use the delete function to delete this file
					self.deleteFile(location)
						
					# Let the user know it worked
					self.infoBar.set_text(os.path.basename(location) + " was moved to ..Wastebasket")
				
				except:
					self.infoBar.set_text("Cannot Remove " + os.path.basename(location))
					return 1
			else:
				# You don't have permission, and so you must know
				self.infoBar.set_text("You don't have permission to move " + os.path.basename(location))
				return 1

		# This is a dangerous command, but necessary
		elif name == "Delete":

			# Use the delete file function
			self.deleteFile(location)

		# Just try and mount, then print any messages output		
		elif name == "Mount":
			try:
				output = commands.getoutput("mount " + location)
				self.infoBar.set_text(output)

				# Relist if we are successful
				self.removeOldIcons()
				self.listCurrentDirectory()

			except:
				# Let the user know what failed
				output = commands.getoutput("mount " + location)
				self.infoBar.set_text(output)				

		# Just try and unmount, then print any messages output
		elif name == "Unmount":
			try:
				output = commands.getoutput("umount " + location)
				self.infoBar.set_text(output)

				# Relist if we are successful
				self.removeOldIcons()
				self.listCurrentDirectory()

			except:
				# Let the user know what failed
				output = commands.getoutput("umount " + location)
				self.infoBar.set_text(output)

		# This attempts to eject the medium
		elif name == "Eject":
			try:
				output = commands.getoutput("eject " + location)
				self.infoBar.set_text(output)

				# Relist if we are successful
				self.removeOldIcons()
				self.listCurrentDirectory()

			except:
				# Let the user know what failed
				output = commands.getoutput("eject " + location)
				self.infoBar.set_text(output)

		# If we want to open the file with something other then listed, then pick it and we'll try
		elif name == "Other":

			# Open a file dialog and get a file
			fileGet = gtk.FileSelection("Open With...")
			fileGet.set_modal(gtk.TRUE)
			response = fileGet.run()
			app = fileGet.get_filename()

			# If cancel or destroy was pressed, then return
			if response != gtk.RESPONSE_OK:
				fileGet.destroy()
				return 1

			# Else destroy the widget, but move on
			fileGet.destroy()

			# Use the execute function 
			self.executeCommand(None, location, app)

		else:
			# Use the execute function 
			self.executeCommand(None, location, name)

		# eventBox.emit("dropped")


	##############################################################
	# This function deletes files (I did it so much I decided to write this function)
	##############################################################
	def deleteFile(self, location):
		
		# If this is a link, we must unlink it
		if os.path.islink(location) and os.access(location, os.W_OK):
			os.unlink(location)

		# If have write permissions on the file and it exists, let's erase it
		elif os.path.isdir(location) and os.access(location, os.W_OK):

			# First let's create a message box to let the user what is happening
			messageBox =  gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, ("Deleting " + os.path.basename(location)))
			messageBox.show()
			try:
				# This is for erasing a directory
				shutil.rmtree(location)

				# Destroy the warning box
				messageBox.destroy()
						
				# Let the user know it worked
				self.infoBar.set_text(os.path.basename(location) + " was deleted")
				
			except:
				# Let the user know if it failed (destroy the message box, too)
				self.infoBar.set_text("Error Deleting " + os.path.basename(location) + ". Check Permissions")
					
				# Destroy the warning box
				messageBox.destroy()
					
				return 1

		# Make sure we have write permissions for this regular file
		elif os.access(location, os.W_OK):

			# First let's create a message box to let the user what is happening
			messageBox =  gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, ("Deleting " + os.path.basename(location)))
			messageBox.show()
				
			try:
				# This is for erasing a regular file
				os.remove(location)

				# Let the user know it worked
				self.infoBar.set_text(os.path.basename(location) + " was deleted")
				
			except:
				# Let the user know if it failed (destroy the message box, too)	
				self.infoBar.set_text("Error Deleting " + os.path.basename(location) + ". Check Permission")
				
			# Destroy the warning box
			messageBox.destroy()
		else:
			# You don't have permission, and so you must know
			self.infoBar.set_text("You don't have permission to move " + os.path.basename(location))
			return 1


	##############################################################
	# This function tries to open a file with an application
	##############################################################
	def executeCommand(self, filetype, location, sent = None):

		# First we must make the location BASH friendly
		location = self.returnClean(location)

		# Let's see if the file is executable (and you double-clicked it)
		if os.access(location, os.X_OK) and sent == None:
			try:
				self.infoBar.set_text("Attempting to execute " + os.path.basename(location))
				os.popen2(location)
				return 0
			except:
				# If it can't execute, then print this message and move on to finding an app
				print "I guess this isn't executable"

		# If no specific app was sent
		if sent == None:

			# Get the associated apps
			applications = mimereader.getApplications(filetype)
			apps = map(lambda app: commands.getoutput("which " + app), applications)
			
			# If there are associated apps, the let's get going
			if apps != []:

				# Now let's use the list to attempt to open the file
				for app in apps:
			
					try:
						# See if we have permission to execute the app
						if os.access(app, os.X_OK):
	
							# Try to open the file with the first app on the list
							os.popen2(app + " " + location)

							# If we succeed, let the user know and exit
							self.infoBar.set_text("Opening " + os.path.basename(location) + " with " + app)
							return 0

						else:
						
							# If we fail permissions, then let the user know
							self.infoBar.set_text("You don't have permission to open " + os.path.basename(location) + " with " + app)
							return 1
					except:
						# If we can't open the file with an app for some reason, then try to execute the file itself
						try:
							os.popen2(location)
						except:
							self.infoBar.set_text("ERROR: Can't open " + os.path.basename(location))

						return 1

		# If this is a special request, get to it
		else:
			try:
				# Try to open what was sent
				os.popen2(sent + " " + location)

				# If we succeed, let the user know and exit
				self.infoBar.set_text("Opening " + os.path.basename(location) + " with " + sent)
				return 0

			except:
				# If we can't open the file with an app for some reason, send an error message
				self.infoBar.set_text("ERROR: Can't open " + os.path.basename(location))		


	##############################################################
	# This is the callback when the pointer is over an icon
	##############################################################
	def overIcon(self, eventBox, event):
		
		# Let other functions know the mouse is over an icon
		self.over = 1

		# Set the layout over variable to the number
		if eventBox in self.currentContents[0]:
			self.layoutOver = 0
		else:
			self.layoutOver = 1	

		#print self.currentContents[self.activeWindow][eventBox][4]

		# Find out which pane we are in
		if eventBox in self.currentContents[0]:
			over = 0
		else:
			over = 1
		
		# Play an audio preview if requested
		if self.previewAudio == "on":
		
			# Set the types and players
			types  = ["mpeg",     "ogg"	,"x-wav",      "basic"]				
			player = ["mpg321", "ogg123" , "playwave", "sox"  ]  

			# Only play something that is an audio file
			if self.currentContents[over][eventBox][3][1] in types:
				number = types.index(self.currentContents[over][eventBox][3][1])
				try:
					os.popen2(player[number] + " " + self.returnClean(self.currentContents[over][eventBox][2]))

					# We set this variable to the player so that it is easy to kill (neat thinking, eh?)
					self.playingMusic = player[number]
					
				# If a valid type can't be played, it's probabaly because the player is not installed
				except:
					self.infoBar.set_text(player[number] + " is not installed")
					self.playingMusic = "no"
				
		# If any step fails, then the music is not on
			else:
				self.playingMusic = "no"
				
		else:
			self.playingMusic = "no"

		# if this is an mp3 but the preview is not on, make a popup window with the file information
		if self.currentContents[over][eventBox][3][1] == "mpeg":

			# Get the file information
			try:
				mp3class = eyeD3.Mp3AudioFile(self.currentContents[over][eventBox][2])			
				mp3tag = eyeD3.Tag()
				mp3tag.link(self.currentContents[over][eventBox][2])
				
				info = {}		
				info["Artist"]  = mp3tag.getArtist()
				info["Title"]  = mp3tag.getTitle()
				info["Album"]  = mp3tag.getAlbum()
				info["Playtime"]  = mp3class.getPlayTimeString()
				info["Bitrate"] = mp3class.getBitRateString()
				info["Size"]  = mp3class.getSize()
			
				# Put the information that exists into an outputtable format
				names = filter(lambda name: info[name] !=  "", info)
				label = ""
				for name in names:
					label += "<b> " + name + "</b> :" + str(info[name]) + "\n"

				# Create a label for the tag info
				tag = gtk.Label()
				tag.set_use_markup(gtk.TRUE)
				tag.set_markup(label)
				tag.show()
						
				# Make a popup window to hold the label
				tagWindow = gtk.Window(gtk.WINDOW_POPUP)
	
				# Place the tag at the mouse location
				x,y = self.mainWindow.get_position()
				offsetx, offsety = self.mainWindow.get_pointer()
				tagWindow.move(x + offsetx+20, y + offsety)

				#  Show and resize the window
				tagWindow.show()
				# tagWindow.resize(tagWindow.allocation.width, tagWindow.alloation.height)

				# Add the icon to the window
				tagWindow.add(tag)

				# Set this to the window
				self.tagpreview =tagWindow			
			
			# If this fails, then there is no preview
			except:
				self.tagpreview = None				
		else:
			self.tagpreview = None
		

		# If this is an image file, show a popup preview
		if self.previewImage == "on" and self.currentContents[over][eventBox][3][0] == "image":

			# If we have permission to read it, then make a pixbuf out of it (we don't have svg or gimp capabilities, yet)
			if os.access(self.currentContents[over][eventBox][2], os.R_OK) and self.currentContents[over][eventBox][3][1] not in ["gimp", "x-3ds"]:

					# We create a pixbuf from the picture (make a copy in memory)
					previewPixbuf = gtk.gdk.pixbuf_new_from_file(self.currentContents[over][eventBox][2])

					# We get the image's original size
					width = previewPixbuf.get_width()
					height = previewPixbuf.get_height()
					wholeWidth = width
					wholeHeight = height

					# Get a scaleable size
					x=1
					while width > 200 or height > 200:
						width = (width*.95)
						height = (height*.95)
						x -= .05

					# Set the height and width of the icon pixbuf
					previewPixbuf = previewPixbuf.scale_simple(int(width), int(height), gtk.gdk.INTERP_BILINEAR)

					# Then we add it to the image object
					preview = gtk.Image()
					preview.show()
					preview.set_from_pixbuf(previewPixbuf)

					# Make a popup window to hold the image
					previewWindow = gtk.Window(gtk.WINDOW_POPUP)

					# Place the image preview at the mouse location
					x,y = self.mainWindow.get_position()
					offsetx, offsety = self.mainWindow.get_pointer()
					previewWindow.move(x + offsetx+20, y + offsety)

					#  Resize it to fit the image
					previewWindow.show()
					previewWindow.resize(int(width)+5, int(height)+5)

					# Add the icon to the window
					previewWindow.add(preview)

					# Set this to the window, similiar to what we did with the audio preview
					self.picpreview = previewWindow

		# Here too, anything but an actual picpreview = None
			else:
				self.picpreview = None
		else:
			self.picpreview = None
	
	
	##############################################################
	# This is the callback when the pointer is not over an icon anymore
	##############################################################	
	def notOverIcon(self, eventBox, event):
		
		# Let other functions know that we are not over an icon (mainly a right click thing)
		self.over = 0

		# If a player is playing music then stop it
		if self.playingMusic != "no":
			os.system("killall " + self.playingMusic)
			self.playingMusic == "no"

		# If a picture preview is showing, destroy it
		if self.picpreview != None:
			self.picpreview.destroy()
			self.picpreview = None

		# If a audio tag is showing, destroy it
		if self.tagpreview != None:
			self.tagpreview.destroy()
			self.tagpreview = None
	

	##############################################################
	# This sets the styles for the icons is selected and unselected states
	##############################################################			
	def iconStateChanged(self, eventBox, state):

		# If the state is normal, set it to selected and highlight the icon
		if state == gtk.STATE_NORMAL:
			iconPixbuf = self.currentContents[self.activeWindow][eventBox][0].get_pixbuf().copy()
			iconPixbuf.saturate_and_pixelate(iconPixbuf, 4.0, gtk.FALSE)
			self.currentContents[self.activeWindow][eventBox][0].set_from_pixbuf(iconPixbuf)

		# If the state is selected, set it to normal and de-highlight the icon
		elif state == gtk.STATE_SELECTED:
			iconPixbuf = self.currentContents[self.activeWindow][eventBox][0].get_pixbuf()
			iconPixbuf.saturate_and_pixelate(iconPixbuf, 0.25, gtk.FALSE)
			self.currentContents[self.activeWindow][eventBox][0].set_from_pixbuf(iconPixbuf)



	##############################################################
	# DRAG AND DROP SECTION
	##############################################################
	
	######################################
	# This function get the currently selected icons
	######################################
	def getSelected(self):
		
		# Make a list to hold the selected icons
		selectedIcons = []

		# Add the selected icons to the list
		selectedIcons = filter(lambda x : x.state == gtk.STATE_SELECTED ,self.currentContents[self.activeWindow])

		# Then return the selections (simple enough)			
		return selectedIcons


	######################################
	# Once a drop is accepted this get the data we need
	######################################
	def dataGet(self, widget, context, selection_data, info, time):

		# Get the selected icons
		selected = self.getSelected()

		# Add the locations of the icons to the list
		passed = ""
		for selection in selected:
			passed += self.currentContents[self.activeWindow][selection][2] + "::"

		# Add the selected locations to the "selection_data" (which is the information passed by the
		# GTK Drag and Drop)
		selection_data.set(selection_data.target, 8, passed)


	######################################
	# This handles a drop onto an layout or an icon
	######################################
	def iconDropped(self, widget, drag_context, x, y, selection_data, info, time):

		# First let's see if the drop was to a layout widget or an icon
		if widget == self.mainLayout[0]:

			# Whatever it is, we get the dropped files
			droppedFiles = string.split(selection_data.data, "::")

			# We then get set the drop location
			dropLocation = self.currentDirectory[0]

		elif widget == self.mainLayout[1]:
			droppedFiles = string.split(selection_data.data, "::")
			dropLocation = self.currentDirectory[1]

		elif widget.parent.parent == self.mainLayout[0]:
			droppedFiles = string.split(selection_data.data, "::")
			dropLocation = self.currentContents[0][widget][2]		

		elif widget.parent.parent == self.mainLayout[1]:
			droppedFiles = string.split(selection_data.data, "::")
			dropLocation = self.currentContents[1][widget][2]	

		# Make sure the drop location is a directory
		if not os.path.isdir(dropLocation):
			self.infoBar.set_text(os.path.basename(dropLocation) + " is not a Directory")
			return 1

		# See if you have permission to drop into the directory
		if not os.access(dropLocation, os.W_OK):
			self.infoBar.set_text("You don't have permission to write to " + dropLocation)
			return 1

		# See if you have permission to read the files you dragged
		for aFile in droppedFiles:
			if not os.access(aFile, os.R_OK):
				droppedFiles.remove(aFile)
				self.infoBar.set_text("You don't have permission to read " +  aFile)

		# Check to see if you are dropping a file ontop of itself (Should we allow this?)
		if dropLocation in droppedFiles:
			place = droppedFiles.index(dropLocation)
			self.infoBar.set_text("You can't drop " + os.path.basename(droppedFiles[place]) + " onto itself. Deselect it first.")
			return 1

		# Lastly, let's see if the file already exists in this directory
		directoryList = os.listdir(dropLocation)
		copies = []
		for name in droppedFiles:
			if os.path.basename(name) in directoryList:

				# If there is another file with the same name, see if they want it overwritten
				answer = choose.choose(self, name)

				# If they don't then add it to the "remove-me-from-the-list" list
				if answer == "no":
					copies.append(name)

				# If they want it overwritten, then remove the old directory or file
				else:
					if os.path.isdir(dropLocation + os.path.basename(name)):
						shutil.rmtree(dropLocation + os.path.basename(name))
					elif os.path.isfile(dropLocation + os.path.basename(name)):
						os.remove(dropLocation + os.path.basename(name))

		# Remove any files we don't want to overwrite
		for acopy in copies:
			droppedFiles.remove(acopy)

		if '' in droppedFiles:
			droppedFiles.remove('')

		# Now that we've done our error checking, let's move onto business

		# First, did any files survive?
		if droppedFiles == []:
			self.infoBar.set_text("Please Select Valid File(s) to Move")
			return 0

		# See if the selection is an http:// address
		if re.findall("http://", droppedFiles[0]) != []:
			self.infoBar.set_text(selection + " is an http:// address!")
			return 0

		# We need to create a menu to give the user options
		menu = gtk.Menu()
		menu.show()

		# Create some menu items (Copy, Link, Move). Make them clickable and pass the
		# name of the sending and receiving widgets
		openItem = gtk.MenuItem("Copy Here")
		openItem.show()
		openItem.connect("activate",self.dropMenu, "Copy", droppedFiles, dropLocation)
		menu.append(openItem)

		openItem = gtk.MenuItem("Link Here")
		openItem.show()
		openItem.connect("activate",self.dropMenu, "Link", droppedFiles, dropLocation)
		menu.append(openItem)

		openItem = gtk.MenuItem("Move Here")
		openItem.show()
		openItem.connect("activate",self.dropMenu, "Move", droppedFiles, dropLocation)
		menu.append(openItem)

		# Make the menu a "popup" (so it appears immediately)
		menu.popup(None, None, None, 0, 0)

 
	######################################
	# This functions handles the callbacks for the drop menu
	######################################
	def dropMenu(self, widget, selection, droppedFiles, dropLocation):


		# Now, let's take care of what was selected
		if selection == "Copy":

			# Use the filemover to copy the files
			for droppedFile in droppedFiles:
				progressdialog.fileMover(self, droppedFile, dropLocation +"/" +  os.path.basename(droppedFile))

			# Let the user know what happened
			self.infoBar.set_text("Files Copied Successfully to " + dropLocation)
			
			return 0
				
		if selection == "Link":

			# Link all the files
			for droppedFile in droppedFiles:
				os.symlink(droppedFile, dropLocation + "/" + os.path.basename(droppedFile))

			# Let the user know what happened
			self.infoBar.set_text("Files Linked Successfully to " + dropLocation) 

		if selection == "Move":

			# Use the filemover to copy the files
			for droppedFile in droppedFiles:
				progressdialog.fileMover(self,  droppedFile, dropLocation +"/" +  os.path.basename(droppedFile))

			# Let the user know what happened
			self.infoBar.set_text("Files Moved Successfully to " + dropLocation)

			# Delete the old file(s)
			for droppedFile in droppedFiles:
			
				# Make sure you have permission to delete the old file
				if os.access(droppedFile, os.W_OK):
					if os.path.isdir(droppedFile):
						shutil.rmtree(droppedFile)
					else:				
						os.remove(droppedFile)
				# If you don't have permission, the you just did a copy (oh, well)
				else:
					self.infoBar.set_text("Can't erase " + droppedFile)
			

	##############################################################
	# INFORMATION / UTILITY SECTION
	##############################################################

	########################################################################
	# The function creates a button and connects it to a signal
	########################################################################	
	def makeButton(self, buttonType, name, signalType, function):
	
		if buttonType == "button":
			button = gtk.Button(name)
		elif buttonType == "toggle":
			button = gtk.ToggleButton(name)
		button.connect(signalType, function)
		button.show()
		return button

	########################################################################
	# Get the users on this computer from /etc/passwd
	########################################################################	
	def getUsers(self):
		infile = file('/etc/passwd', 'r').readlines()

		self.users = {}

		for line in infile:
			line = string.split(line,':')
			self.users[line[2]] = line[0]


	########################################################################
	# Get the informationf or a passed file
	########################################################################	
	def getFileInfo(self, aFile):
		
		# If this is a non-existant file or dead link, then let's return some bugus values
		if not os.path.exists(aFile):		
			return "0 bytes", 'Dead Link', "Unknown"	

		# Make some variables
		permissions=""

		# Get the file information
		try:
			fileInfo = os.lstat(aFile)

			# Get the UID and then use the username dictionary to get the user name
			UID = fileInfo[stat.ST_UID]
			user = self.users[str(UID)]

			# Check to make sure the user is valid
			if user == None or user == "":
				user = "unknown"

			# Get the file size
			if fileInfo[stat.ST_SIZE] > 1024:
				fileSize = str(fileInfo[stat.ST_SIZE]/1024) + " Kbs"
			else:
				fileSize = str(fileInfo[stat.ST_SIZE]) + " bytes"

			# Get the permissions in octal form (UNIX numeric [777,755,530] form)
			mode = oct(fileInfo[stat.ST_MODE])

			# This dictionary contains the permission types
			permissionTypes = {7:'rwx', 6:'rw-', 5:'r-x', 4:'r--', 3:'-wx', 2:'-w-', 1:'--x', 0:'---'}

			# Determine the permissions
			for number in mode[-3:]:
				permissions += permissionTypes[int(number)]

		except:
			# If we can't get the file information from stat, we'll get it the old fashioned way
			user = "Unknown"
			fileSize = os.path.getsize(aFile)
			
			# Get the file size
			if fileSize > 1024:
				fileSize = str(fileSize/1024) + " Kbs"
			else:
				fileSize = str(fileSize) + " bytes"	

			# This is the old way of getting permissions
			if os.access(aFile, os.R_OK):
				permissions += "r"
			else:
				permissions += "-"
			if os.access(aFile, os.W_OK):
				permissions += "w"
			else:
				permissions += "-"
			if os.access(aFile, os.X_OK):
				permissions += "x"
			else:
				permissions += "-"

		# Return whatever we get
		return fileSize, permissions, user


	########################################################################
	# This function returns BASH friendly text
	########################################################################		
	def returnClean(self, text):
	
		# Here are the bad characters
		badChars = [" ","\'","&","(",")"]
		
		# Here are the replacement characters
		goodChars=["\ ","\\'","\&","\(","\)"]
		
		# Replace the bad with the good
		for x in range(len(badChars)):
			text = string.replace(text, badChars[x], goodChars[x])		
		
		# Return the "cleaned", BASH friendly text
		return text


	################################################################
	# This is the mainloop, which listens for signals and sends them to "connect"-ed widgets 
	################################################################
	def main(self):
		gtk.main()


if __name__=="__main__":
	claw = Claw3()
	claw.main()	
