#!/usr/bin/env python

########################################################################
# This is the mimetype reader for the CLAW3 Filebrowser
# Copyright 2004, Gabe Ginorio <gabe@zevallos.com.br>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
########################################################################

import string, re, os, os.path, commands
import pygtk, gtk, pango


########################################################################
# This function gets the applications associated with an icon
########################################################################	
def getApplications(mime):

	# Get the group and filetype from the mime information
	group, extention = mime[0], mime[1]

	# Open the appropriate mime-file
	mimefile = file(os.environ['HOME'] + "/.claw/mime." + group).readlines()
	
	# Get the appropriate line
	line = filter(lambda line: re.findall("^" + extention, string.strip(string.split(line, "::")[0])), mimefile)
	
	# Check to see if there was a match
	if line != []:
	
		# Get the apps fron the line (one big mess, but there are tabs and spaces to deal with)
		applications = string.strip(string.split(line[0], "::")[1])
		applications = string.split(applications, ",")
		applications = map(lambda app: string.strip(app), applications)
	else:
		# If there wasn't a match, then send back nothing and let the user know
		applications = []
		print "-------------------------------------"
		print mime, "is not is the mime files"
		print "-------------------------------------"	

	# Then we need to see if the apps actually exists
	applications = filter(lambda app: commands.getstatusoutput("which " + string.split(app)[0])[0] == 0, applications)

	# Return the appropriate information
	return applications


########################################################################
# This function updates any installed mime files in ~/.claw
########################################################################		
def update(Claw3):

	# Create a list of the mime files we expect to find
	mimes = ["application","audio","image","text","types","video"]
	
	# Open each file in sequence
	for mime in mimes:

		# Read the file that came with the Claw3
		mimeOutFile = 	file(Claw3.programPath + "mimetypes/mime." + mime, 'r')
		mimeOutLines = mimeOutFile.readlines()
		mimeOutFile.close()
		
		# Read the file that is in ~/.claw
		mimeInFile	 =	file(os.environ['HOME'] + "/.claw/mime." + mime, 'r')
		mimeInLines =  mimeInFile.readlines()
		mimeInFile.close()

		# Get all the extentions
		outExtentions = map(lambda line: string.strip(string.split(line, "::")[0]), mimeOutLines)
		inExtentions   = map(lambda line: string.strip(string.split(line, "::")[0]), mimeInLines)

		# Get the extention If the extention doesn't exist in the ~/.claw mime file
		differentExtentions = filter(lambda outExtention: outExtention not in inExtentions, outExtentions)
		
		# Get the whole line of any missing extentions
		differentLines = []
		if differentExtentions != []:
			for extention in differentExtentions:
				for line in mimeOutLines:
					if extention in string.strip(string.split(line, "::")[0]):
						differentLines.append(line)

		# Get all lines in the Claw3 version not in the ~/.claw version
		# differentLines = filter(lambda line: line not in mimeInLines, mimeOutLines)

		# Reopen the ~/.claw file for appending
		mimeInFile	 =	file(os.environ['HOME'] + "/.claw/mime." + mime, 'a')
		
		# Add any lines that are different
		if differentLines != []:
			map(lambda line: mimeInFile.write(line), differentLines)
		
		# Close the file and we are done with this file
		mimeInFile.close()
		

########################################################################
# This function returns BASH friendly text
########################################################################		
def returnClean(text):
	
		# Here are the bad characters
		badChars = [" ","\'","&","(",")"]
		
		# Here are the replacement characters
		goodChars=["\ ","\\'","\&","\(","\)"]
		
		# Replace the bad with the good
		for x in range(len(badChars)):
			text = string.replace(text, badChars[x], goodChars[x])		
		
		# Return the "cleaned", BASH friendly text
		return text




########################################################################
# This function creates the mime editor dialog
########################################################################	
def editor(Claw3, eventBox):

	# Extract the relevant information from the current directory's information table
	location 	= Claw3.currentContents[Claw3.activeWindow][eventBox][2]
	group		= Claw3.currentContents[Claw3.activeWindow][eventBox][3][0]
	extention	= Claw3.currentContents[Claw3.activeWindow][eventBox][3][1]	

	# This will hold the entries
	global entries

	# Create a window for the editor
	mimeWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
	mimeWindow.set_title("File Association Dialog")
	mimeWindow.connect("destroy", quit, mimeWindow)
	
	# Create a Table to format information in the mime editor window
	mimeTable = gtk.Table(gtk.FALSE)
	mimeWindow.add(mimeTable)
	
	# Create a Frame for the icon picture association 
	iconFrame = gtk.Frame("Associated Icon")
	mimeTable.attach(iconFrame, 0,1,0,1, gtk.FILL, gtk.SHRINK)

	# Create an image using the associated icon
	iconImage = gtk.Image()
	
	# Remember that non xcf and svg images are picture previews
	if group == "image" and extention not in ["gimp", "x-3ds"]:
		iconPixbuf = gtk.gdk.pixbuf_new_from_file(location)	
		iconPixbuf = iconPixbuf.scale_simple(48, 48, gtk.gdk.INTERP_BILINEAR)	
		iconImage.set_from_pixbuf(iconPixbuf)
	else:
		iconImage.set_from_file(Claw3.imagePath + "icon/" + extention + ".png")
	iconFrame.add(iconImage)
	
	# Create a Frame for the extention
	extentionFrame = gtk.Frame("Associated Extention")
	mimeTable.attach(extentionFrame, 0, 1, 1, 2, gtk.FILL, gtk.SHRINK)
	
	# Create a Label for the extention
	extentionLabel = gtk.Label(extention)
	extentionFrame.add(extentionLabel)

	# Create a Frame for the applications
	applicationsFrame = gtk.Frame("Associated Applications")
	mimeTable.attach(applicationsFrame, 0, 1, 2, 3, gtk.FILL, gtk.SHRINK)
	
	# Create a box to hold the applicatin entries
	applicationsBox = gtk.VBox()
	applicationsFrame.add(applicationsBox)
	
	# Get the applications with the mime type and format them for use
	applications = getApplications(Claw3.currentContents[Claw3.activeWindow][eventBox][3])
	if len(applications) > 0:
		applications = map(lambda app: string.strip(app), applications)
	
	# This will hold the entry widgets
	global entries
	entries = []
	
	# Create the entries and editing buttons for the associated applications	
	# I created a function for this so we could add entry boxes later
	map(lambda app: createEntry(None, app, applicationsBox, Claw3) ,applications)

	# Create a box to hold the add, save and cancel buttons
	buttonBox = gtk.HBox()
	mimeTable.attach(buttonBox, 0, 1, 3, 4)

	# Create the add button
	addButton = gtk.Button("Add")
	addButton.connect("clicked", createEntry,  "", applicationsBox, Claw3)
	buttonBox.pack_start(addButton)

	# Create the save button
	saveButton = gtk.Button("Save")
	saveButton.connect("clicked", save, mimeWindow, Claw3, eventBox)
	buttonBox.pack_start(saveButton)

	# Create the cancel button	
	cancelButton = gtk.Button("Cancel")
	cancelButton.connect("clicked", quit, mimeWindow)
	buttonBox.pack_start(cancelButton)						
							
	# Now, we show the widget all at once						
	mimeWindow.show_all()


#######################################
# This function creates an entry
######################################
def createEntry(widget, app, applicationsBox, Claw3):

		# Create a box for the entries and buttons
		entryBox = gtk.HBox()
		applicationsBox.pack_start(entryBox)
	
		# Create the entry
		entry = gtk.Entry()
		entry.set_text(app)
		# entry.set_state(gtk.STATE_INSENSITIVE)
		entry.set_editable(gtk.FALSE)
		entryBox.pack_start(entry)
		
		# Add the entry to the entries list
		entries.append(entry)

		# Create the find button
		findButton = gtk.Button()
		findImage = gtk.Image()
		findImage.set_from_file(Claw3.imagePath + "other/find.png")
		findButton.add(findImage)
		findButton.connect("clicked", find, entry)
		entryBox.pack_start(findButton)		
		
		# Create the edit button
		editButton = gtk.Button()
		editImage = gtk.Image()
		editImage.set_from_file(Claw3.imagePath + "other/edit.png")
		editButton.add(editImage)
		editButton.connect("clicked", edit, entry)
		entryBox.pack_start(editButton)		
		
		# Create the delete button
		deleteButton = gtk.Button()
		deleteImage = gtk.Image()
		deleteImage.set_from_file(Claw3.imagePath + "other/delete.png")
		deleteButton.add(deleteImage)
		deleteButton.connect("clicked", delete, entryBox)
		entryBox.pack_start(deleteButton)
		
		# This is only for if a box is added
		entryBox.show_all()



#######################################
# This function is the callback to find an app button
#######################################
def find(findButton, entry):
		
		# Create a file dialog
		fileGet = gtk.FileSelection("Locate Application")
		
		# Set it to modal (the main window freezes while this is open)
		fileGet.set_modal(gtk.TRUE)
		
		# Run the dialog
		response = fileGet.run()
		
		# Get the returned application
		application = fileGet.get_filename()

		# Destroy the widget when donw
		fileGet.destroy()

		# If OK was not pressed then do nothing
		if response != gtk.RESPONSE_OK:
			return 1

		# Otherwise, see if the app is valid and you can execute it
		if os.path.exists(application) and os.access(application, os.X_OK):
			
			# Get the system application search path
			paths = string.split(os.environ['PATH'], ":")
			
			# See if the application is in the system path
			path = filter(lambda x : os.path.dirname(application) == x, paths)
			
			# If the application is in the path, remove the leading directories
			if path != []:
				application = string.replace(application, path[0] + "/", "")
			
			# Set the entry text to the application location/name
			entry.set_text(application)


#######################################
# This function is the callback to edit an entry
#######################################
def edit(editButton, entry):

	# Make the entry editable (or not editable)
	if entry.get_editable() == gtk.TRUE:
		entry.set_editable(gtk.FALSE)
	else:
		entry.set_editable(gtk.TRUE)	
	

#######################################
# This function is the callback to delete an entry
#######################################
def delete(deleteButton, entryBox):

	# First, we delete the entry box from the entries list
	entries.remove(entryBox.get_children()[0])

	# Then we delete the box whole box containing the entry and buttons
	entryBox.destroy()


#######################################
# This function is the callback to save the new mimetype
#######################################
def save(button, mimeWindow, Claw3, eventBox):

	# Here's some information we'll need
	group		= Claw3.currentContents[Claw3.activeWindow][eventBox][3][0]
	extention	= Claw3.currentContents[Claw3.activeWindow][eventBox][3][1]	

	# Get the text in the entry boxes
	apps = map(lambda x: x.get_text(), entries)
	apps = map(lambda x: string.strip(x), apps)
	
	# Make a comma sererated string of the associated applications
	text = None
	if apps != []:
		text = apps[0]
		if len(apps) > 1:
			for app in apps[1:]:
				text += "," + app 

	# Open the appropriate mime-file and read it
	mimefile = file( os.environ['HOME'] + "/.claw/mime." + group, 'r')
	lines = mimefile.readlines()
	mimefile.close()

	# Open the appropriate mime-file for writing
	mimefile = file( os.environ['HOME'] + "/.claw/mime." + group, 'w')
	
	# Edit the appropriate line
	for line in lines:
		if re.findall(string.replace(extention,".png",""), string.split(line, "::")[0]) != []:
			ext, apps = string.split(line, "::")
			line = ext + "::\t" + text + "\n"
		
		mimefile.write(line)
	mimefile.close()	

	# Quit the mime editor
	quit(button, mimeWindow)


#######################################
# This function is the callback to destroy the app
#######################################
def quit(button, mimeWindow):

	# Destroy the window
	mimeWindow.destroy()

	
