#!/usr/bin/python

########################################################################
# This is the preference setter for the CLAW3 Filebrowser
# Copyright 2004, Gabe Ginorio <gabe@zevallos.com.br>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
######################################################################

import os, re, string, sys

import pygtk
pygtk.require("2.0")
import gtk, pango

from xml.dom import minidom

class Dialog:

	def __init__(self, Claw3):
		
		# Create the main window, give it a title, connect it to destroy and size it
		self.dialogWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialogWindow.show()
		self.dialogWindow.set_title("CLAW3 Preference Setter")
		self.dialogWindow.connect("destroy", self.quit)

		# Place the preference dialog window at the mouse location
                x,y = Claw3.mainWindow.get_position()
                offsetx, offsety = Claw3.mainWindow.get_pointer()
                self.dialogWindow.move(x + offsetx+20, y + offsety)

		# Make a table to hold everything
		mainTable = gtk.Table()
		mainTable.show()
		self.dialogWindow.add(mainTable)

		# Make a box for the logo and main label
		mainLogo = gtk.Image()
		mainLogo.show()
		mainTable.attach(mainLogo, 0, 1, 0, 1, gtk.FILL, gtk.FILL)

		# Make a note book to separate the preferences
		dialogNotebook = gtk.Notebook()
		dialogNotebook.show()
		mainTable.attach(dialogNotebook, 0, 1, 1, 2)

		# Make a table to hold the general preferences
		self.dialogTable = gtk.Table()
		self.dialogTable.show()

		# Make a table to hold the style preferences
		self.styleTable = gtk.Table()
		self.styleTable.show()

		# Make some labels for the note book and add some pages
		generalLabel = gtk.Label("General")
		generalLabel.modify_font(pango.FontDescription("sans bold 12"))
		generalLabel.show()
		dialogNotebook.append_page(self.dialogTable, generalLabel)

		styleLabel = gtk.Label("Style")
		styleLabel.modify_font(pango.FontDescription("sans bold 12"))
		styleLabel.show()
		dialogNotebook.append_page(self.styleTable, styleLabel)

		# This function reads the preferences file
		preferences = self.readPreferencesFile(Claw3)

		# This function creates the interface and sets it according to the preferences
		self.makeInterface(preferences, Claw3)

		# Make a box for the control buttons
		controlBox = gtk.HBox()
		controlBox.show()
		mainTable.attach(controlBox, 0, 1, 2, 3, gtk.FILL, gtk.FILL)

		# Create some buttons so we can get out of here
		saveButton = gtk.Button("Save")
		saveButton.show()
		saveButton.get_child().modify_font(pango.FontDescription("sans bold 12"))
		saveButton.connect("clicked", self.save, Claw3)
		controlBox.pack_start(saveButton)
		controlBox.get_children()[0].fill = gtk.TRUE

		cancelButton = gtk.Button("Cancel")
		cancelButton.show()
		cancelButton.get_child().modify_font(pango.FontDescription("sans bold 12"))
		cancelButton.connect("clicked", self.quit)
		controlBox.pack_start(cancelButton)
		controlBox.get_children()[1].fill = gtk.TRUE


	#########################################################
	# This function reads the ~/.claw/preferences.xml file (and makes sure it is there)
	#########################################################
	def readPreferencesFile(self, Claw3):

		# First, is the file there, or do we need to send a copy (it could happen)
		if not os.path.exists(os.environ['HOME'] + "/.claw/preferences3.xml"):

			try:
				# Put a copy of the default file in the ~/.claw/ directory
				shutil.copy(Claw3.programPath + "preferences.xml", os.environ['HOME'] + "/.claw/preferences3.xml")			
			except:
				# If this fails, then print and error and exit
				Claw3.infoBar.set_text("Cannot get preferences.xml from " + string.replace(sys.argv[0], "Claw3.py", ""))
				self.quit()

		# Secondly, let's see if the file is in error or it is good
		try:
			preferenceFile = minidom.parse(os.environ['HOME'] + "/.claw/preferences3.xml")
		except:
			# If this fails, then print and error and exit
			Claw3.infoBar.set_text("Cannot read ~/.claw/preferences3.xml. Correct it or erase it.")
			self.quit()			


		# Now, if it opened, let's get the preferences from it
		
		# Let's set a list of items we expect to find in the file
		preferenceItems = ["terminal", "clear_address", "image", "audio", "permissions", "archiver", 
				"font", "color", "background","language"]
	
		# Make a list to hold the stuff we get from the file
		preferences = []

		# Get the information from the xml (DOM) tree
		try:
			for item in preferenceItems:
				node = preferenceFile.getElementsByTagName(item)
				preferences.append(node[0].childNodes[0].data)

			# Return the preference list
			return preferences

		# If there is an error in the file itself (something missing) let the user know, and quit
		except:
			Claw3.infoBar.set_text("Cannot parce ~/.claw/preferences3.xml properly. Correct it or erase it.")
			self.quit()	


	#########################################################
	# This function creates the widget interface
	#########################################################
	def makeInterface(self, preferences, Claw3):

		# Create some boxes for the preferences
		boxes = []
		for x in range(8):
			boxes.append(gtk.VBox())
			boxes[x].show()
			self.dialogTable.attach(boxes[x], 0, 1, x, x+1, gtk.FILL, gtk.FILL)

		# Create a frame for the terminal preference
		self.terminalFrame = gtk.Frame("Terminal Command")
		self.terminalFrame.show()
		boxes[0].pack_start(self.terminalFrame)

		# Create an entry for the terminal preference	
		self.terminalEntry = gtk.Entry()
		self.terminalEntry.show()
		self.terminalEntry.modify_font(pango.FontDescription("sans bold 12"))
		self.terminalFrame.add(self.terminalEntry)

		
		# Create a "Use Default" checkbox
		self.terminalCheckBox = gtk.CheckButton("Use Default Terminal")
		self.terminalCheckBox.show()
		boxes[0].pack_start(self.terminalCheckBox)
		
		self.terminalEntry.set_text(preferences[0])
		self.oldterm = preferences[0]
		
		if preferences[0] == "default":
			self.terminalCheckBox.set_active(gtk.TRUE)
			self.terminalEntry.set_sensitive(gtk.FALSE)
			
		# If the default button is checked, the entry box is insensitive	
		self.terminalCheckBox.connect("toggled", self.setDefaultTerm)

		# Create a frame for the archiver preference	
		archiverFrame = gtk.Frame("Select Archiver")
		archiverFrame.show()
		boxes[1].pack_start(archiverFrame)

		# Create a combobox to hold the archiver selections
		self.archiverCombo = gtk.Combo()
		self.archiverCombo.show()
		self.archiverCombo.set_popdown_strings(["bzip2","gzip","zip -r"])
		self.archiverCombo.entry.set_editable(gtk.FALSE)
		self.archiverCombo.entry.modify_font(pango.FontDescription("sans bold 12"))
		self.archiverCombo.entry.set_text(preferences[5])
		archiverFrame.add(self.archiverCombo)

		# Create a frame for the on/off preferences
		buttonFrame = gtk.Frame("Simple Browsing Preferences")
		buttonFrame.show()
		boxes[2].pack_start(buttonFrame)

		# Create a box to put inside the on/off frame
		buttonBox = gtk.VButtonBox()
		buttonBox.show()
		buttonFrame.add(buttonBox)

		# Create a button for the image previews
		self.imageButton = gtk.CheckButton("Show Image Previews")
		self.imageButton.get_child().modify_font(pango.FontDescription("sans bold 12"))
		self.imageButton.get_child().modify_fg(gtk.STATE_ACTIVE, gtk.gdk.Color(0,40000,0,10))
		self.imageButton.get_child().modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(40000,0,0,10))
		self.imageButton.show()
		buttonBox.pack_start(self.imageButton)

		# Set the state of the button depending on what is in the preference file
		if preferences[2] == "on":
			self.imageButton.set_active(gtk.TRUE)
		else:
			self.imageButton.set_active(gtk.FALSE)

		# Create a button for the audio previews
		self.audioButton = gtk.CheckButton("Hear Audio Previews")
		self.audioButton.show()
		self.audioButton.get_child().modify_font(pango.FontDescription("sans bold 12"))
		self.audioButton.get_child().modify_fg(gtk.STATE_ACTIVE, gtk.gdk.Color(0,40000,0,10))
		self.audioButton.get_child().modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(40000,0,0,10))
		buttonBox.pack_start(self.audioButton)

		# Set the state of the button depending on what is in the preference file
		if preferences[3] == "on":
			self.audioButton.set_active(gtk.TRUE)
		else:
			self.audioButton.set_active(gtk.FALSE)

		# Create a button for permission showing
		self.permissionButton = gtk.CheckButton("Show File Permissions")
		self.permissionButton.show()
		self.permissionButton.get_child().modify_font(pango.FontDescription("sans bold 12"))
		self.permissionButton.get_child().modify_fg(gtk.STATE_ACTIVE, gtk.gdk.Color(0,40000,0,10))
		self.permissionButton.get_child().modify_fg(gtk.STATE_NORMAL, gtk.gdk.Color(40000,0,0,10))
		buttonBox.pack_start(self.permissionButton)

		# Set the state of the button depending on what is in the preference file
		if preferences[4] == "yes":
			self.permissionButton.set_active(gtk.TRUE)
		else:
			self.permissionButton.set_active(gtk.FALSE)


		###########################################################
                
		# Make a frame for the font selections and add it to tab2
		self.fontFrame = gtk.Frame("Font Selections")
		self.fontFrame.show()
		self.styleTable.attach(self.fontFrame, 0, 1, 0, 1)
                
		# Make a box for the font selections and add it ot the frame
		self.fontBox = gtk.VBox()
		self.fontBox.show()
		self.fontFrame.add(self.fontBox)

		# Make a icon font secection
		self.iconFont = preferences[6] 
		self.iconLabel = gtk.Label(self.iconFont)
		self.icondesc = pango.FontDescription(self.iconFont)      
		self.iconLabel.modify_font(self.icondesc)
		self.iconLabel.show()
		self.fontBox.pack_start(self.iconLabel)

		# Make a button for the font selection
		self.fontButton = gtk.Button("Choose Font")
		self.fontButton.show()            
		self.fontButton.connect("clicked", self.selectFont)
		self.fontBox.pack_start(self.fontButton)	

		# Make a button to set the font color
		self.colorButton = gtk.Button("Font Color")
		self.colorButton.show()           
		self.colorButton.connect("clicked", self.selectColor)
		self.fontBox.pack_start(self.colorButton)       

		# Set the font color
		colors = string.split(preferences[7], ",")
		self.fontColor = gtk.gdk.Color(int(colors[0]), int(colors[1]), int(colors[2]), 0)
		self.iconLabel.modify_fg(gtk.STATE_NORMAL, self.fontColor)

		# Make a frame for the background selection
		self.backgroundFrame = gtk.Frame("Background")
		self.backgroundFrame.show()
		self.styleTable.attach(self.backgroundFrame, 0, 1, 1, 2)
                
		# Make a box for the background selection and add it ot the frame
		self.backgroundBox = gtk.VBox()
		self.backgroundBox.show()
		self.backgroundFrame.add(self.backgroundBox)
		
		# Make a button to set the font color
		self.backgroundButton = gtk.Button("Background Color")
		self.backgroundButton.show()           
		self.backgroundButton.connect("clicked", self.selectBackground, Claw3)
		self.backgroundBox.pack_start(self.backgroundButton) 

		# Set the background color
		colors = string.split(preferences[8], ",")
		self.backgroundColor = gtk.gdk.Color(int(colors[0]), int(colors[1]), int(colors[2]), 0)
		
		# Make a frame for the language selction
		languageFrame = gtk.Frame("Select Language")
		languageFrame.show()
		boxes[3].pack_start(languageFrame)
		boxes[3].show()
		
		# Make a combobox to hold the language selections
		self.languageCombo = gtk.Combo()
		self.languageCombo.show()
		languageFrame.add(self.languageCombo)

		languages = os.listdir(Claw3.programPath + "translations")
		languages = map(lambda language : string.replace(language, ".trans", "") , languages)
		self.languageCombo.set_popdown_strings(languages)
		self.languageCombo.entry.set_text(preferences[9])
		self.languageCombo.entry.set_editable(gtk.FALSE)
		

		#########################################################################################

		# Make a frame for the theme selection
		themeFrame = gtk.Frame("Gtk Theme")
		themeFrame.show()
		self.styleTable.attach(themeFrame, 0, 1, 2, 3)
                
		# Make a box for the theme selection and add it ot the frame
		themeBox = gtk.VBox()
		themeBox.show()
		themeFrame.add(themeBox)

		# Make a combo box to hold the themes available
		self.themeCombo = gtk.Combo()
		self.themeCombo.show()
		themeBox.pack_start(self.themeCombo)

		# Get the gtk-2.0 themes in /usr/share/themes/
		themes = os.listdir("/usr/share/themes")

		# Get the gtk-2.0 themes in ~/.themes/
		if os.path.exists(os.environ['HOME'] + "/.themes"):
			userThemes = os.listdir(os.environ['HOME'] + "/.themes")
		else:
			userThemes = []

		# Add the user themes to the system themes
		themes = themes + userThemes

		# Keep only the ones that exist and contain gtk-2.0 directories
		themes = filter(lambda x: os.path.exists("/usr/share/themes/" + x + "/gtk-2.0") or os.path.isdir(os.environ['HOME'] + "/.themes/" + x + "/gtk-2.0") , themes)

		# Add the available themes to the combo box
		self.themeCombo.set_popdown_strings(themes)

		# Now we need to get the current gtk style (if a ~/.gtkrc file exists)
		if os.path.exists(os.environ['HOME'] + "/.gtkrc"):

			# Open the .gtkrc file
			rcFile = file(os.environ['HOME'] + "/.gtkrc", 'r').readlines()
			
			# Find the line that points to a theme and make it readable to humans
			for line in rcFile:
				if re.findall("/gtk-2.0/", line):
					line = string.replace(line, "include", "")
					line = string.replace(line, "\"", "")
					line = string.replace(line, "/gtk-2.0/gtkrc", "")
					line = os.path.basename(line)
					line = string.strip(line)
					self.currentTheme = line

					#set the selected to the current theme
					self.themeCombo.entry.set_text(self.currentTheme)

		# No editing the entries!!!
		self.themeCombo.entry.set_editable(gtk.FALSE)

		# Connect an event to the box so that the theme will change on demand
		self.themeCombo.entry.connect("changed", self.changeTheme, Claw3)


	############################################
	# The Callback for checking the default terminal button
	############################################   
	def setDefaultTerm(self, checkbutton):

		# Check and change the state
		if checkbutton.get_active():
			self.terminalEntry.set_text("default")
			self.terminalEntry.set_sensitive(gtk.FALSE)
		else:
			if self.oldterm == "default":
				self.terminalEntry.set_text("xterm")
			else:
				self.terminalEntry.set_text(self.oldterm)
			self.terminalEntry.set_sensitive(gtk.TRUE)
		

	############################################
	# The Callback for clicking the Select Font Button
	############################################                
        def selectFont(self, button):
                
                # Open a new font dialog
                self.fontdialog = gtk.FontSelectionDialog("Select Icon Font")
		self.fontdialog.set_font_name(self.iconFont)
		self.fontdialog.set_preview_text("Thank you for using the CLAW3 File Browser!")
                name = self.fontdialog.run()
                
                # Get the response from the dialog 
                if name == gtk.RESPONSE_OK:
                    
                    # If it's an OK, the get the selected font
                    self.iconFont = self.fontdialog.get_font_name()
                    
                    # Put the font in the dialog box
                    self.iconLabel.set_text(self.iconFont)
                    font_desc=pango.FontDescription(self.iconFont)
                    self.iconLabel.modify_font(font_desc)
                    self.iconLabel.set_text(self.iconFont)
                
                self.fontdialog.destroy()


	############################################
	# The Callback for selecting a font color
	############################################
        def selectColor(self, button):

                # Open a new color dialog
                color = gtk.gdk.Color(10000,10000,10000,0)
                self.colordialog = gtk.ColorSelectionDialog("Select Font Color")
                self.colordialog.colorsel.set_current_color(color)
                name = self.colordialog.run()
                
                # Get the response from the dialog 
                if name == gtk.RESPONSE_OK:
                    
                    # If it's an OK, the get the selected color
                    self.fontColor = self.colordialog.colorsel.get_current_color()
                    
                    # Make the font preview the appropriate the color
                    self.iconLabel.modify_fg(gtk.STATE_NORMAL, self.fontColor)
                
                self.colordialog.destroy()     

		
	############################################
	# The Callback for selecting a background color
	############################################
        def selectBackground(self, button, Claw3):

                # Open a new color dialog
                self.colordialog = gtk.ColorSelectionDialog("Select Font Color")
                self.colordialog.colorsel.set_current_color(Claw3.layoutColor)
                name = self.colordialog.run()
                
		# Get the response from the dialog 
		if name == gtk.RESPONSE_OK:
                    
			# If it's an OK, the get the selected color
			self.backgroundColor = self.colordialog.colorsel.get_current_color()
                    
			# Change the claw layout backgrounds to the selected color
			Claw3.mainLayout[0].modify_bg(gtk.STATE_NORMAL, self.backgroundColor)
			Claw3.mainLayout[1].modify_bg(gtk.STATE_NORMAL, self.backgroundColor)

			# Change the icon backgrounds
			for icon in Claw3.currentContents[0]:
				icon.modify_bg(gtk.STATE_NORMAL, self.backgroundColor)
			for icon in Claw3.currentContents[1]:
				icon.modify_bg(gtk.STATE_NORMAL, self.backgroundColor)
                
                self.colordialog.destroy()    

	############################################
	# The Callback for selecting a theme
	############################################
	def changeTheme(self, button, Claw3):

		# Get the selected theme
		theme = self.themeCombo.entry.get_text()

		# If the theme exists, load it
		if theme != "":

			# Change the line in the ~/.gtkrc file
			rcFile = file(os.environ['HOME'] + "/.gtkrc", 'w')
			rcFile.write("# This is the default gtk-2.0 theme\n")
			rcFile.write("include \"/usr/share/themes/" + theme + "/gtk-2.0/gtkrc\"")
			rcFile.close()

			# Parse the theme
			gtk.rc_parse(os.environ['HOME'] + "/.gtkrc")

			# Redraw the widget
			Claw3.mainWindow.reset_rc_styles()

			# Make sure the widget is redrawn
			while  gtk.events_pending():
				gtk.main_iteration()


	#########################################################
	# This function saves the new preferences
	#########################################################
	def save(self, button, Claw3):		

		# Get the information from the widget
		terminal = self.terminalEntry.get_text()	
		archiver = self.archiverCombo.entry.get_text()

		audio = self.audioButton.get_property("active")
		if audio == gtk.TRUE:
			audio = "on"
		else:
			audio = "off"

		image = self.imageButton.get_property("active")
		if image == gtk.TRUE:
			image = "on"
		else:
			image = "off"	

		permissions = self.permissionButton.get_property("active")
		if permissions == gtk.TRUE:
			permissions = "yes"
		else:
			permissions = "no"
			
		language = self.languageCombo.entry.get_text()

		# Open the preferences file and read it
		preferencesFile = file(os.environ['HOME'] + "/.claw/preferences3.xml", 'r')
		lines = preferencesFile.readlines()
		preferencesFile.close()

		# Open the file for writing
		preferencesFile = file(os.environ['HOME'] + "/.claw/preferences3.xml", 'w')

		# Replace the relevant lines
		for line in lines:
			if re.findall("terminal", line):
				line = "\t<terminal>"+terminal+"</terminal>\n"
			elif re.findall("archiver", line):
				line = "\t<archiver>"+archiver+"</archiver>\n"
			elif re.findall("image", line):
				line = "\t<image>"+image+"</image>\n"
			elif re.findall("permissions", line):
				line = "\t<permissions>"+permissions+"</permissions>\n"
			elif re.findall("audio", line):
				line = "\t<audio>"+audio+"</audio>\n"
			elif re.findall("font", line):
				line = "\t<font>"+self.iconFont+"</font>\n"
			elif re.findall("color", line):
				line = "\t<color>"+str(self.fontColor.red) + "," + str(self.fontColor.green) + "," + str(self.fontColor.blue) + "</color>\n"
			elif re.findall("background", line):
				line = "\t<background>"+str(self.backgroundColor.red) + "," + str(self.backgroundColor.green) + "," + str(self.backgroundColor.blue) + "</background>\n"
			elif re.findall("language", line):
				line = "\t<language>"+ language + "</language>\n"
			# Write the line		
			preferencesFile.write(line)

		# Close the File
		preferencesFile.close()

		# Set the preferences and move on
		if terminal == "default":
			Claw3.termType = os.getenv('TERMCMD', "xterm")
		else:
			Claw3.termType = terminal			
		Claw3.previewImage	= string.strip(image)
		Claw3.previewAudio	= string.strip(audio)
		Claw3.showPermissions	= string.strip(permissions)
		Claw3.archiver		= string.strip(archiver)
		Claw3.font 			= pango.FontDescription(self.iconFont)
		Claw3.fontColor		= gtk.gdk.Color(self.fontColor.red, self.fontColor.green, self.fontColor.blue, 0)
		Claw3.layoutColor	= self.backgroundColor
		# Set the background color

		Claw3.mainLayout[0].modify_bg(gtk.STATE_NORMAL, Claw3.layoutColor)
		Claw3.mainLayout[1].modify_bg(gtk.STATE_NORMAL, Claw3.layoutColor)

		# Do a relist
		if Claw3.secondScroll.get_property('visible'):
			Claw3.activeWindow=1
			Claw3.removeOldIcons()
			Claw3.listCurrentDirectory()
		Claw3.activeWindow=0
		Claw3.removeOldIcons()
		Claw3.listCurrentDirectory()
		Claw3.language = language

		# Quit the app
		self.quit()


	#########################################################
	# This function destroys the widget
	#########################################################
	def quit(self, button=None):
		self.dialogWindow.destroy()
