#!/usr/bin/env python


################################################################################
# This is the progress dialog for the CLAW3 file browser
# Copyright 2004, Gabe Ginorio <gabe@zevallos.com.br>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
 #   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

import os, os.path, sys, shutil
import gtk, pango


class fileMover:
	def __init__(self, Claw3, start, end):

		# Create the main window and name it after the file being moved
		self.progressWindow = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.progressWindow.connect('destroy', self.kill)
		self.progressWindow.set_title("The CLAW3 File Mover")

		# Place the progress window at the mouse location
                x,y = Claw3.mainWindow.get_position()
                offsetx, offsety = Claw3.mainWindow.get_pointer()
                self.progressWindow.move(x + offsetx+20, y + offsety)     

		# Create a box for the dialog (in case I want to add more stuff later)
		self.mainTable = gtk.Table(1, 2, gtk.FALSE)
		self.mainTable.show()
		self.progressWindow.add(self.mainTable)

		# Create a Label for the Dialog (to hold the file name)
		self.mainLabel = gtk.Label("Moving " + os.path.basename(start))
		# self.mainLabel.show()
		self.mainLabel.modify_font(pango.FontDescription("sans bold 12"))
		self.mainTable.attach(self.mainLabel, 0,1,0,1)

		# Create the progress bar and add it to the box
		self.progressBar = gtk.ProgressBar()
		self.progressBar.show()
		self.mainTable.attach(self.progressBar, 0,1,1,2)

		# Set the window size and show all
		self.progressWindow.resize(self.progressWindow.allocation.width, 50)
		self.progressWindow.show_all()

		# Find out if this is a directory or a file 
		if os.path.isdir(start):
			self.moveDirectory(start, end, Claw3)
		else:
			# Start the file moving
			self.moveFile(start, end, Claw3)
			
		self.kill()

	###################################################################
	# This function opens the file and moves it piece by piece, showing the progress
	# in the progress bar
	###################################################################
	def moveFile(self, start, end, Claw3):

		# Get the size of the file to move
		filesize = os.path.getsize(start)
		
		# If the file size isn't zero
		if filesize != 0:
			
			# Get the size of the file chunks (needs to be 100 of them)
			chunkSize = int(round(filesize/100))

			# However, if the chunksize < 0 (under 1K files), set the size to 1 byte
			if chunkSize == 0:
				chunkSize = 1
		else:
			Claw.infoBar.set_text("This file is empty")
			return 1

		# Open the file we want to move
		infile = file(start, 'r')
		
		# Open a new file for writing to with the 
		outfile = file(end, 'w')

		# Set some variables
		movedAmount, movedPercent = 0, 0
		
		# if the moved amount isn't the size of the original file, then keep going
		while not movedAmount >= filesize:
			
			# Read a chunk from the original and write it to the new one
			fileChunk = infile.read(chunkSize)
			outfile.write(fileChunk)

			# Resize the window so the name fits
			width, height = self.progressBar.size_request()
			self.progressWindow.set_size_request(width, 25)

			# Increment the variables (% and amount moved)
			movedPercent+= .01
			movedAmount+=len(fileChunk)

			# Update the progress bar 
			if movedPercent < 1.0:
 				self.progressBar.set_fraction(movedPercent)
			if movedAmount < 1000:
				self.progressBar.set_text(str(movedAmount) +" bytes")
			else:
				self.progressBar.set_text(str(movedAmount/1024) +" Kbs")

			# This function allows the gui to update in case the machine is too busy
			self.updategui()

		# Close both files
		infile.close()
		outfile.close()

		# These lines are just for testing to make sure the entire file transferred
		# print os.path.getsize(start)
		# print os.path.getsize(end)
	
	###################################################################
	# This function moves the contents of directories
	###################################################################
	def moveDirectory(self, start, end, Claw3):

		# First we need the number of files in the directory
		files = os.listdir(start)

		# Get the number of files listed and then get a good increment for the progress bar
		amount = len(files)
		if amount != 0:
			increment = 1.0/amount
		else:
			increment = 1.0

		# If the directory is empty, then just move it and return
		if len(files) == 0:
			shutil.copytree(start, end)
			return 0

		# Here are some necessary variables
		movedPercent = 0

		# Otherwise, get the file or files in the directory and move them
		try:
			os.mkdir(end)
		except:
			print "Directory Exists"

		for aFile in files:

			# Update the progress Bar
			self.progressBar.set_fraction(movedPercent)
			self.progressBar.set_text("Moving " + aFile)

			# Resize the window so the name fits
			width, height = self.progressBar.size_request()
			self.progressWindow.set_size_request(width, 25)
			

			# This function allows the gui to update in case the machine is too busy
			self.updategui()

			# Copy to actual file or folder
			if os.path.isdir(start + "/" + aFile):
				try:
					shutil.copytree(start + "/" + aFile, end + "/" + aFile)
				except:
					Claw3.infoBar.set_text("Error Copying " + start + "/" + aFile)
			else:
				shutil.copy(start + "/" + aFile, end + "/")     

			# Increment the percentage of files moved
			movedPercent += increment


	###################################################################
	# This function makes all current processes end before allowing it to continue
	# This allows the GUI to update even when the processor is peaked
	###################################################################
	def updategui(self):
		while  gtk.events_pending() :
			gtk.main_iteration()
		return


	###################################################################
	# This function destroys the widget when we are finished
	###################################################################
	def kill(self, button=None):
		self.progressWindow.destroy()
